@@ -641,6 +641,11 @@ void btn_load_custom_pressed(void)
 	playlist_dialog_add_custom_type(FALSE);
 }
 
+void btn_save_pressed(void)
+{
+	playlist_dialog_save_as(playlist_pathname);
+}
+
 /*
  *-----------------------------------------------------------------------------
  * preset callbacks
