@@ -279,6 +279,26 @@ static void button_mixer_click_cb(ButtonData *button, const gchar *key, gpointer
 	btn_mixer_pressed();
 }
 
+static void button_open_file_click_cb(ButtonData *button, const gchar *key, gpointer data)
+{
+	btn_load_file_pressed();
+}
+
+static void button_open_playlist_click_cb(ButtonData *button, const gchar *key, gpointer data)
+{
+	btn_load_playlist_pressed();
+}
+
+static void button_open_custom_click_cb(ButtonData *button, const gchar *key, gpointer data)
+{
+	btn_load_custom_pressed();
+}
+
+static void button_save_playlist_click_cb(ButtonData *button, const gchar *key, gpointer data)
+{
+	btn_save_pressed();
+}
+
 static void button_preset_click_cb(ButtonData *button, const gchar *key, gpointer data)
 {
 	gint n;
@@ -789,6 +809,30 @@ void display_register_widgets(UIData *ui)
 			    NULL, NULL,
 			    NULL, NULL);
 
+	button_register_key("open_file", ui,
+			    NULL, NULL,
+			    button_open_file_click_cb, NULL,
+			    NULL, NULL,
+			    NULL, NULL);
+
+	button_register_key("open_playlist", ui,
+			    NULL, NULL,
+			    button_open_playlist_click_cb, NULL,
+			    NULL, NULL,
+			    NULL, NULL);
+
+	button_register_key("open_custom", ui,
+			    NULL, NULL,
+			    button_open_custom_click_cb, NULL,
+			    NULL, NULL,
+			    NULL, NULL);
+
+	button_register_key("save_playlist", ui,
+			    NULL, NULL,
+			    button_save_playlist_click_cb, NULL,
+			    NULL, NULL,
+			    NULL, NULL);
+
 	for (i = 0; i < 10; i++)
 		{
 		gchar *buf;
