@@ -414,10 +414,16 @@ static void handle_warnings(gchar *text)
 	if (strncmp(text, "Directory:", 10) == 0) return;
 	if (strncmp(text, "Getting real-time priority", 26) == 0) return;
 
+	/* mpg123 0.59s-pre */
+	if (strncmp(text, "Found new ID3 Header", 20) == 0) return;
+
 	/* for freeamp in mpg123 compat mode */
 	if (strncmp(text, "Copyright", 9) == 0) return;
 	if (strncmp(text, "This soft", 9) == 0) return;
 
+	/* Tru64's mpg123 version reports the device it uses on start */
+	if (strncmp(text, "Using device", 12) == 0) return;
+
 	/* ignore lines that begin with an escape sequence (0.59r) */
 	if (text[0] == 27 && text[1] == ']') return;
 
@@ -442,6 +448,9 @@ static void pipe_reader_shutdown(void)
 
 	if (mpg123_pipe_read_id != -1) gtk_input_remove(mpg123_pipe_read_id);
 	mpg123_pipe_read_id = -1;
+
+	close(mpg123_pipe[0]);
+	mpg123_pipe[0] = -1;
 }
 
 static void stop_data(void)
@@ -642,10 +651,13 @@ static gint start_playing_file(SongData *sd, gint position)
 	pid_t frk_pid;
 	char cmd_arguments[MPG123_MAX_COMMANDS][512];
 	char *cmd_ptr[MPG123_MAX_COMMANDS];
+	size_t cmd_max;
 	int cmd_cnt = 0;
 	gint start_frame;
 	gchar *exec_bin = MPG123_BINARY;
 
+	cmd_max = sizeof(cmd_arguments) / MPG123_MAX_COMMANDS;
+
 	if (position == 0)
 		{
 		start_frame = 0;
@@ -664,27 +676,27 @@ static gint start_playing_file(SongData *sd, gint position)
 
 	/* create all command line arguments */
 
-	strcpy(cmd_arguments[cmd_cnt], MPG123_BINARY);
+	strncpy(cmd_arguments[cmd_cnt], MPG123_BINARY, cmd_max);
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
 
-	strcpy(cmd_arguments[cmd_cnt], "-v");
+	strncpy(cmd_arguments[cmd_cnt], "-v", cmd_max);
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
 
-	strcpy(cmd_arguments[cmd_cnt], "-k");
+	strncpy(cmd_arguments[cmd_cnt], "-k", cmd_max);
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
-	sprintf(cmd_arguments[cmd_cnt], "%d", start_frame);
+	snprintf(cmd_arguments[cmd_cnt], cmd_max, "%d", start_frame);
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
 
 	if (mpg123_buffer_enable && !mpg123_to_wav)
 		{
-		strcpy(cmd_arguments[cmd_cnt], "-b");
+		strncpy(cmd_arguments[cmd_cnt], "-b", cmd_max);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
-		sprintf(cmd_arguments[cmd_cnt], "%d", mpg123_buffer);
+		snprintf(cmd_arguments[cmd_cnt], cmd_max, "%d", mpg123_buffer);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 		}
@@ -693,19 +705,19 @@ static gint start_playing_file(SongData *sd, gint position)
 		{
 		if (mpg123_downsample == MPG123_DOWNSAMPLE_22)
 			{
-			strcpy(cmd_arguments[cmd_cnt], "-2");
+			strncpy(cmd_arguments[cmd_cnt], "-2", cmd_max);
 			cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 			cmd_cnt++;
 			}
 		if (mpg123_downsample == MPG123_DOWNSAMPLE_11)
 			{
-			strcpy(cmd_arguments[cmd_cnt], "-4");
+			strncpy(cmd_arguments[cmd_cnt], "-4", cmd_max);
 			cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 			cmd_cnt++;
 			}
 		if (mpg123_downsample == MPG123_DOWNSAMPLE_CUSTOM)
 			{
-			sprintf(cmd_arguments[cmd_cnt], "-r %d", mpg123_custom_sample_size);
+			snprintf(cmd_arguments[cmd_cnt], cmd_max, "-r %d", mpg123_custom_sample_size);
 			cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 			cmd_cnt++;
 			}
@@ -713,38 +725,38 @@ static gint start_playing_file(SongData *sd, gint position)
 
 	if (mpg123_mono)
 		{
-		strcpy(cmd_arguments[cmd_cnt], "-m");
+		strncpy(cmd_arguments[cmd_cnt], "-m", cmd_max);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 		}
 
 	if (mpg123_8bit)
 		{
-		strcpy(cmd_arguments[cmd_cnt], "--8bit");
+		strncpy(cmd_arguments[cmd_cnt], "--8bit", cmd_max);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 		}
 
 	if (mpg123_aggressive)
 		{
-		strcpy(cmd_arguments[cmd_cnt], "--aggressive");
+		strncpy(cmd_arguments[cmd_cnt], "--aggressive", cmd_max);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 		}
 
 	if (mpg123_disable_resync)
 		{
-		strcpy(cmd_arguments[cmd_cnt], "--resync");
+		strncpy(cmd_arguments[cmd_cnt], "--resync", cmd_max);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 		}
 
 	if (mpg123_device_enable && mpg123_device && !mpg123_to_wav)
 		{
-		strcpy(cmd_arguments[cmd_cnt], "-a");
+		strncpy(cmd_arguments[cmd_cnt], "-a", cmd_max);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
-		strcpy(cmd_arguments[cmd_cnt], mpg123_device);
+		strncpy(cmd_arguments[cmd_cnt], mpg123_device, cmd_max);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 		}
@@ -756,14 +768,14 @@ static gint start_playing_file(SongData *sd, gint position)
 
 		if (mpg123_to_raw)
 			{
-			strcpy(cmd_arguments[cmd_cnt], "--cdr");
+			strncpy(cmd_arguments[cmd_cnt], "--cdr", cmd_max);
 			cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 			cmd_cnt++;
 			ext = ".raw";
 			}
 		else
 			{
-			strcpy(cmd_arguments[cmd_cnt], "-w");
+			strncpy(cmd_arguments[cmd_cnt], "-w", cmd_max);
 			cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 			cmd_cnt++;
 			ext = ".wav";
@@ -781,7 +793,7 @@ static gint start_playing_file(SongData *sd, gint position)
 			buf = g_strconcat(tmp, ".wav", NULL);
 			g_free(tmp);
 			}
-		strcpy(cmd_arguments[cmd_cnt], buf);
+		strncpy(cmd_arguments[cmd_cnt], buf, cmd_max);
 		cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 		cmd_cnt++;
 
@@ -800,7 +812,7 @@ static gint start_playing_file(SongData *sd, gint position)
 
 		while(vector && vector[i] != NULL)
 			{
-			strcpy(cmd_arguments[cmd_cnt], vector[i]);
+			strncpy(cmd_arguments[cmd_cnt], vector[i], cmd_max);
 			cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 			cmd_cnt++;
 			i++;
@@ -812,17 +824,17 @@ static gint start_playing_file(SongData *sd, gint position)
 		{
 		gchar *buf;
 		buf = http_path_ensure_root(sd->path);
-		strcpy(cmd_arguments[cmd_cnt], buf);
+		strncpy(cmd_arguments[cmd_cnt], buf, cmd_max);
 		g_free(buf);
 		}
 	else
 		{
-		strcpy(cmd_arguments[cmd_cnt], sd->path);
+		strncpy(cmd_arguments[cmd_cnt], sd->path, cmd_max);
 		}
 	cmd_ptr[cmd_cnt] = cmd_arguments[cmd_cnt];
 	cmd_cnt++;
 
-	strcpy(cmd_arguments[cmd_cnt], "");
+	strncpy(cmd_arguments[cmd_cnt], "", cmd_max);
 	cmd_ptr[cmd_cnt] = NULL;
 	cmd_cnt++;
 
@@ -864,6 +876,7 @@ static gint start_playing_file(SongData *sd, gint position)
 		pid = (int) frk_pid;
 		fcntl(mpg123_pipe[0], F_SETFL, O_NONBLOCK);
 		close(mpg123_pipe[1]);
+		mpg123_pipe[1] = -1;
 		}
 
 	if (debug_mode) printf("mpg123 pid = %d\n", pid);
@@ -1313,6 +1326,7 @@ void mpg123_init(void)
 	module_register_file_suffix_type(".mp2", mp3_desc, id);
 	module_register_file_suffix_type(".mpeg", mp3_desc, id);
 	module_register_file_suffix_type(".mpg", mp3_desc, id);
+	module_register_file_suffix_type(".m2a", mp3_desc, id);
 	module_register_misc_type(_("MP3/Shoutcast stream"), "http://server:port", id, TRUE,
 			type_is_http, mpg123_http_entry_setup, mpg123_http_edit, mpg123_http_get_path);
 
