@@ -46,6 +46,81 @@ static gint ogg123_control_read_id = -1;
 static gint ogg123_child_pid = -1;
 static int ogg123_pipe[2];
 
+
+/*
+ *-----------------------------------------------------------------------------
+ * pipe/fork util
+ *-----------------------------------------------------------------------------
+ */
+
+static gint ogg123_pipe_open(char *ogg_argv[], gint *ogg_child, int ogg_pipe[2])
+{
+	pid_t frk_pid;
+
+	/* Create the pipe. */
+	if (pipe(ogg_pipe))
+		{
+		fprintf (stderr, _("Pipe failed.\n"));
+		return FALSE;
+		}
+
+	if (debug_mode) printf("ogg123_pipe_open: %s\n", ogg_argv[0]);
+
+	/* Create the child process. */
+	frk_pid = fork ();
+	if (frk_pid == (pid_t) 0)
+		{
+		/* This is the child process. */
+		dup2(ogg_pipe[1], 1);
+		dup2(ogg_pipe[1], 2);
+		close(ogg_pipe[0]);
+
+		execvp(ogg_argv[0], ogg_argv);
+		printf(_("unable to run %s (in the path?)\n"), ogg_argv[0]);
+		_exit(1);
+		}
+	else if (frk_pid < (pid_t) 0)
+		{
+		/* The fork failed. */
+		fprintf (stderr, _("Fork failed.\n"));
+		close(ogg_pipe[0]);
+		close(ogg_pipe[1]);
+		*ogg_child = -1;
+		return FALSE;
+		}
+	else
+		{
+		/* This is the parent process. */
+		*ogg_child = (int) frk_pid;
+
+		close(ogg_pipe[1]);
+		ogg_pipe[1] = -1;
+		}
+
+	if (debug_mode) printf("ogg123_pipe_open pid = %d\n", *ogg_child);
+
+	return TRUE;
+}
+
+static FILE *ogg123_pipe_open_fd(char *ogg_argv[], gint *ogg_child, int ogg_pipe[2])
+{
+	FILE *f;
+
+	if (!ogg123_pipe_open(ogg_argv, ogg_child, ogg_pipe))
+		{
+		printf("Failed to run \"%s\n\"", ogg_argv[0]);
+		return NULL;
+		}
+	f = fdopen(ogg_pipe[0], "r");
+	if (!f)
+		{
+		close(ogg_pipe[0]);
+		return NULL;
+		}
+
+	return f;
+}
+
 /*
  *----------------------------------------------------------------------------
  * title/description utils
@@ -100,20 +175,19 @@ OGGInfo *ogg123_info_get(const gchar *path)
 {
 	OGGInfo *info = NULL;
 	FILE *f;
-	gchar *command;
+	char *command[3];
+	gint child_pid;
+	int child_pipe[2];
 	char buf[128];
 
 	if (!ogg123_info_found) return NULL;
 
-	command = g_strdup_printf("%s \"%s\"", OGG123_BINARY_INFO, path);
-	f = popen(command, "r");
-	if (!f)
-		{
-		printf("Failed to run \"%s\n\"", command);
-		g_free(command);
-		return NULL;
-		}
-	g_free(command);
+	command[0] = OGG123_BINARY_INFO;
+	command[1] = (char *)path;
+	command[2] = NULL;
+
+	f = ogg123_pipe_open_fd(command, &child_pid, child_pipe);
+	if (!f) return NULL;
 
 	info = g_new0(OGGInfo, 1);
 
@@ -163,7 +237,7 @@ OGGInfo *ogg123_info_get(const gchar *path)
 			}
 		}
 
-	pclose(f);
+	fclose(f);	/* also closes child_pipe[0] */
 
 	info->bytes = filesize(path);
 
@@ -178,20 +252,20 @@ GList *ogg123_comment_get(const gchar *path)
 {
 	GList *list = NULL;
 	FILE *f;
-	gchar *command;
+	char *command[4];
+	gint child_pid;
+	int child_pipe[2];
 	char buf[128];
 
 	if (!ogg123_comment_found) return NULL;
 
-	command = g_strdup_printf("%s -l \"%s\"", OGG123_BINARY_COMMENT, path);
-	f = popen(command, "r");
-	if (!f)
-		{
-		printf("Failed to run \"%s\n\"", command);
-		g_free(command);
-		return NULL;
-		}
-	g_free(command);
+	command[0] = OGG123_BINARY_COMMENT;
+	command[1] = "-l";
+	command[2] = (char *)path;
+	command[3] = NULL;
+
+	f = ogg123_pipe_open_fd(command, &child_pid, child_pipe);
+	if (!f) return NULL;
 
 	while (fgets(buf, sizeof(buf), f) != NULL)
 		{
@@ -214,7 +288,7 @@ GList *ogg123_comment_get(const gchar *path)
 		g_free(key);
 		}
 
-	pclose(f);
+	fclose(f);	/* also closes child_pipe[0] */
 
 	return g_list_reverse(list);
 }
@@ -250,17 +324,41 @@ const gchar *ogg123_comment_value(GList *list, const gchar *key)
 gint ogg123_comment_set(const gchar *path, GList *comments)
 {
 	FILE *f;
-	gchar *command;
 	GList *errlist;
+	GList *comment_list;
 	GList *work;
 	char buf[128];
-	gchar *tmp;
+	int argc;
+	char **argv;
+	gint child_pid;
+	int child_pipe[2];
 	
 	if (!ogg123_comment_found) return TRUE;
 	if (!path) return FALSE;
 
-	command = g_strdup_printf("%s -w \"%s\"", OGG123_BINARY_COMMENT, path);
+	comment_list = NULL;
+	errlist = NULL;
 
+	/* get the size needed for argv */
+	work = comments;
+	argc = 4;
+	while (work && work->next)
+		{
+		if (work->data) argc+=2;
+		work = work->next;
+		work = work->next;
+		}
+
+	argv = g_malloc0(argc * sizeof(argv[0]));
+	argc = 0;
+	argv[argc] = OGG123_BINARY_COMMENT;
+	argc++;
+	argv[argc] = "-w";
+	argc++;
+	argv[argc] = (char *)path;
+	argc++;
+
+	/* fill comment arguments */
 	work = comments;
 	while (work && work->next)
 		{
@@ -272,38 +370,33 @@ gint ogg123_comment_set(const gchar *path, GList *comments)
 
 		if (key && strlen(key) > 0 && value && strlen(value) > 0)
 			{
-			tmp = g_strdup_printf("%s -t \"%s=%s\"", command, key, value);
-			g_free(command);
-			command = tmp;
+			argv[argc] = "-t";
+			argc++;
+			argv[argc] = g_strdup_printf("%s=%s", key, value);
+			/* remember allocated strings for later free'ing */
+			comment_list = g_list_prepend(comment_list, argv[argc]);
+			argc++;
 			}
 
 		work = work->next;
 		work = work->next;
 		}
-	tmp = g_strdup_printf("%s 2>&1", command);
-	g_free(command);
-	command = tmp;
 
-	f = popen(command, "r");
-	if (!f)
-		{
-		printf("Failed to run \"%s\"\n", command);
-		g_free(command);
-		return FALSE;
-		}
-	g_free(command);
+	argv[argc] = NULL;
 
-	errlist = NULL;
-	while (fgets(buf, sizeof(buf), f) != NULL)
+	f = ogg123_pipe_open_fd(argv, &child_pid, child_pipe);
+	if (f)
 		{
-		if (strlen(buf) > 0)
+		while (fgets(buf, sizeof(buf), f) != NULL)
 			{
-			errlist = g_list_append(errlist, g_strdup(buf));
+			if (strlen(buf) > 0)
+				{
+				errlist = g_list_append(errlist, g_strdup(buf));
+				}
 			}
+		fclose(f);
 		}
 
-	pclose(f);
-
 	if (errlist)
 		{
 		GenericDialog *gd;
@@ -336,6 +429,7 @@ gint ogg123_comment_set(const gchar *path, GList *comments)
 		}
 
 	path_list_free(errlist);
+	path_list_free(comment_list);
 	return (errlist == NULL);
 }
 
@@ -480,11 +574,34 @@ static void parse_bitrate(const gchar *text)
 	gint force = FALSE;
 
 	ptr = strstr(text, "Bitrate: ");
+	if (ptr)
+		{
+		/* 1.0 rc1 and rc2 */
+		ptr += 9;
+		}
+	else
+		{
+		/* 1.0 rc3 (and newer?) */
+		ptr = strstr(text, "kbps");
+		if (ptr)
+			{
+			while (ptr > text && *ptr != '(') ptr--;
+			if (*ptr == '(')
+				{
+				ptr++;
+				}
+			else
+				{
+				ptr = NULL;
+				}
+			}
+		}
+
+	/* ptr should now be pointing directly at the number now (or NULL) */
 	if (ptr)
 		{
 		gint new_rate;
 
-		ptr += 9;
 		new_rate = (gint)strtol(ptr, NULL, 10);
 		if (input_bitrate != 0)
 			{
@@ -511,6 +628,16 @@ static gint ogg123_input_parse(const gchar *buffer)
 {
 	if (debug_mode > 1) printf("ogg123 output:\"%s\"\n", buffer);
 
+/* Note: different releases have different output formats...
+ *
+ * 1.0 rc1 and rc2:
+ *			Time: 00:01.50 [04:32.13] of 04:33.63, Bitrate: 122.3
+ * 1.0 rc3:
+ *			Time: 00:03.92 [04:48.17] of 04:52.09  (140.9 kbps)  Output Buffer 75.0%
+ *
+ * as of right now this only effects parse_bitrate()
+ */
+
 	if (strncmp(buffer, "Time: ", 6) == 0)
 		{
 		const gchar *ptr;
@@ -648,12 +775,10 @@ static void ogg123_input_read_reset(void)
 
 static gint ogg123_child_run(SongData *sd, gint position)
 {
-	pid_t frk_pid;
 	char cmd_arguments[OGG123_MAX_COMMANDS][512];
 	char *cmd_ptr[OGG123_MAX_COMMANDS];
 	int cmd_cnt = 0;
 	size_t cmd_max;
-	gchar *exec_bin = OGG123_BINARY;
 
 	cmd_max = sizeof(cmd_arguments) / OGG123_MAX_COMMANDS;
 
@@ -720,48 +845,12 @@ static gint ogg123_child_run(SongData *sd, gint position)
 	cmd_ptr[cmd_cnt] = NULL;
 	cmd_cnt++;
 
-	/* Create the pipe. */
-	if (pipe(ogg123_pipe))
-                {
-                fprintf (stderr, _("Pipe failed.\n"));
-                return FALSE;
-                }
-
-        if (debug_mode) printf("opening: %s\n", sd->path);
-
-	/* Create the child process. */
-	frk_pid = fork ();
-	if (frk_pid == (pid_t) 0)
+	pid = 0;
+	if (!ogg123_pipe_open(cmd_ptr, &ogg123_child_pid, ogg123_pipe))
 		{
-		/* This is the child process. */
-		dup2(ogg123_pipe[1], 2);
-		close(ogg123_pipe[0]);
-
-		execvp(exec_bin, cmd_ptr);
-		printf(_("unable to run %s (in the path?)\n"), exec_bin);
-		_exit(1);
-		}
-	else if (frk_pid < (pid_t) 0)
-		{
-		/* The fork failed. */
-		fprintf (stderr, _("Fork failed.\n"));
-		close(ogg123_pipe[0]);
-		close(ogg123_pipe[1]);
-		pid = 0;
-		ogg123_child_pid = -1;
 		return FALSE;
 		}
-	else
-		{
-		/* This is the parent process. */
-		ogg123_child_pid = (int) frk_pid;
-		pid = ogg123_child_pid;
-
-		close(ogg123_pipe[1]);
-		ogg123_pipe[1] = -1;
-		}
-
-	if (debug_mode) printf("ogg123 pid = %d\n", ogg123_child_pid);
+	pid = ogg123_child_pid;
 
 	ogg123_input_read_reset();
 	ogg123_control_read_id = gdk_input_add (ogg123_pipe[0], GDK_INPUT_READ, ogg123_input_read_cb, NULL);
@@ -793,7 +882,7 @@ static gint ogg123_data_set(SongData *sd, gint read_info)
 		if (info)
 			{
 			sd->length = info->length;
-			sd->bit_rate = info->bitrate_nominal;
+			sd->bit_rate = info->bitrate_average / 1000;
 			sd->channels = info->channels;
 			sd->bit_depth = 16;	/* these two correct ? */
 			sd->khz_rate = 44;
