@@ -850,7 +850,8 @@ void help_window_show(const gchar *key)
 
 	help_window = help_window_new(_("Help - GQmpeg"), "GQmpeg", "help",
 				      GQMPEG_SKINDIR "/README", key);
-	gtk_signal_connect(GTK_OBJECT(help_window), "destroy", help_window_destroy_cb, NULL);
+	gtk_signal_connect(GTK_OBJECT(help_window), "destroy",
+			   GTK_SIGNAL_FUNC(help_window_destroy_cb), NULL);
 }
 
 static void show_help(void)
@@ -1118,7 +1119,6 @@ int main (int argc, char *argv[])
 			else
 				{
 				printf(_("unknown option: %s\nuse --help for options\n"),argv[i]);
-				return 0;
 				}
 			g_free(command_line_path);
 			i++;
