@@ -14,6 +14,7 @@
 #include "menus.h"
 
 #include "btn_funcs.h"
+#include "playlist.h"
 #include "playlist-window.h"
 #include "preferences.h"
 #include "ui_help.h"
@@ -130,6 +131,9 @@ GtkWidget *menu_main(UIData *ui)
 	menu_item_add(submenu, _("Custom..."), GTK_SIGNAL_FUNC(btn_load_custom_pressed), NULL);
         gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
 
+	item = menu_item_add_sensitive(menu, _("Save playlist..."), (playlist_get_count() > 0),
+				       btn_save_pressed, NULL);
+
 	menu_item_add_divider(menu);
 
 	item = menu_item_add(menu, _("Toggle"), NULL, NULL);
