@@ -14,7 +14,7 @@
 #define MENUS_H
 
 
-GtkWidget *menu_main(void);
+GtkWidget *menu_main(UIData *ui);
 GtkWidget *menu_filelist_dir(void);
 GtkWidget *menu_filelist_file(void);
 GtkWidget *menu_filelist_playlist(void);
