@@ -42,12 +42,12 @@ static void close_song_info_window(GtkWidget *widget, gpointer data)
 static void save_tag_pressed(GtkWidget *w, gpointer data)
 {
 	SongInfoDialog *s = data;
-        gchar *title;
-        gchar *artist;
-        gchar *album;
-	gchar *year;
-	gchar *comment;
-	gchar *genre_text;
+        const gchar *title;
+        const gchar *artist;
+        const gchar *album;
+	const gchar *year;
+	const gchar *comment;
+	const gchar *genre_text;
 	guint8 genre = 0;
 
         title = gtk_entry_get_text(GTK_ENTRY(s->title_entry));
@@ -225,16 +225,16 @@ GtkWidget *mpg_create_info_window(const gchar *file)
 	if (d || h)
 		{
 		s->title_entry = songinfo_add_entry(vbox1, _("Title"), d ? d->title : NULL,
-						    30, data_changed_cb, s);
+						    30, GTK_SIGNAL_FUNC(data_changed_cb), s);
 		s->artist_entry = songinfo_add_entry(vbox1, _("Artist"), d ? d->artist : NULL,
-						     30, data_changed_cb, s);
+						     30, GTK_SIGNAL_FUNC(data_changed_cb), s);
 		s->album_entry = songinfo_add_entry(vbox1, _("Album"), d ? d->album : NULL,
-						    30, data_changed_cb, s);
+						    30, GTK_SIGNAL_FUNC(data_changed_cb), s);
 		s->year_entry = songinfo_add_entry(vbox1, _("Year"), d ? d->year : NULL,
-						   4, data_changed_cb, s);
+						   4, GTK_SIGNAL_FUNC(data_changed_cb), s);
 		s->genre_combo = add_combo_info(vbox1, _("Genre"), d ? d->genre : 255, s);
 		s->comment_entry = songinfo_add_entry(vbox1, _("Comment"), d ? d->comment : NULL,
-						      30, data_changed_cb, s);
+						      30, GTK_SIGNAL_FUNC(data_changed_cb), s);
 
 		hbox = gtk_hbutton_box_new();
 		gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
