@@ -1134,17 +1134,17 @@ void plw_playlist_randomize_pressed(void)
  *-----------------------------------------------------------------------------
  */
 
-static void dirlist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gint dirlist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 {
 	gint row = -1;
 	gint column = -1;
-	if (bevent->window != GTK_CLIST(widget)->clist_window) return;
+	if (bevent->window != GTK_CLIST(widget)->clist_window) return FALSE;
 
 	gtk_clist_get_selection_info (GTK_CLIST (widget), bevent->x, bevent->y, &row, &column);
 	if (row == -1 || column == -1)
 		{
 		gtk_object_set_user_data(GTK_OBJECT(dirlist_clist), GINT_TO_POINTER(row));
-		return;
+		return FALSE;
 		}
 
 	if (bevent->button == 1 && bevent->type == GDK_2BUTTON_PRESS)
@@ -1152,6 +1152,7 @@ static void dirlist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer
 		gchar *newdir = dirlist_get_path(row);
 		change_to_path(newdir);
 		g_free(newdir);
+		return TRUE;
 		}
 	if (bevent->button == 2 || bevent->button == 3)
 		{
@@ -1167,6 +1168,8 @@ static void dirlist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer
 			       bevent->button, bevent->time);
 		
 		}
+
+	return FALSE;
 }
 
 /*
@@ -1175,20 +1178,21 @@ static void dirlist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer
  *-----------------------------------------------------------------------------
  */
 
-static void filelist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gint filelist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 {
 	gint row = -1;
 	gint column = -1;
-	if (bevent->window != GTK_CLIST(widget)->clist_window) return;
+	if (bevent->window != GTK_CLIST(widget)->clist_window) return FALSE;
 
 	gtk_clist_get_selection_info (GTK_CLIST (widget), bevent->x, bevent->y, &row, &column);
-	if (row == -1 || column == -1) return;
+	if (row == -1 || column == -1) return FALSE;
 
 	if (bevent->button == 1 && bevent->type == GDK_2BUTTON_PRESS)
 		{
 		gchar *newfile = filelist_get_path(row);
 		playlist_add(newfile, TRUE);
 		g_free(newfile);
+		return TRUE;
 		}
 	if (bevent->button == 2 || bevent->button == 3)
 		{
@@ -1220,6 +1224,7 @@ static void filelist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointe
 			       bevent->button, bevent->time);
 		}
 
+	return (bevent->button == 2 || bevent->button == 3);
 }
 
 /*
@@ -1228,20 +1233,21 @@ static void filelist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointe
  *-----------------------------------------------------------------------------
  */
 
-static void playlist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gint playlist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 {
 	gint row = -1;
 	gint column = -1;
-	if (bevent->window != GTK_CLIST(widget)->clist_window) return;
+	if (bevent->window != GTK_CLIST(widget)->clist_window) return FALSE;
 
 	gtk_clist_get_selection_info (GTK_CLIST (widget), bevent->x, bevent->y, &row, &column);
-	if (row == -1 || column == -1) return;
+	if (row == -1 || column == -1) return FALSE;
 
 	if (debug_mode)	printf("%s\n", playlist_get_item(row));
 
 	if (bevent->button == 1 && bevent->type == GDK_2BUTTON_PRESS)
 		{
 		current_song_set_and_play(row, NULL);
+		return TRUE;
 		}
 	if (bevent->button == 2 || bevent->button == 3)
 		{
@@ -1267,6 +1273,19 @@ static void playlist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointe
 		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
 			       bevent->button, bevent->time);
 		}
+
+	return (bevent->button == 2 || bevent->button == 3);
+}
+
+static void playlist_select_row_cb(GtkCList *clist, gint row, gint column,
+				   GdkEvent *event, gpointer data)
+{
+	SongData *sd;
+
+	sd = playlist_get_data(row);
+	if (!sd) return;
+
+	playlist_update_background(sd);
 }
 
 /*
@@ -1377,29 +1396,29 @@ static void update_history_menu(gchar *path)
 		gchar *path;
 		gchar *hist_trunc;
 		hist_trunc = truncate_hist_text(hist,32);
-		button = gtk_menu_item_new_with_label (hist_trunc);
+		button = gtk_menu_item_new_with_label(hist_trunc);
 		g_free(hist_trunc);
 
 		path = g_strdup(hist);
 		history_list = g_list_append(history_list, path);
-		gtk_signal_connect (GTK_OBJECT (button), "activate",
-			(GtkSignalFunc) history_menu_select, path);
+		gtk_signal_connect(GTK_OBJECT(button), "activate",
+				   GTK_SIGNAL_FUNC(history_menu_select), path);
 
-		gtk_menu_append (GTK_MENU (historymenu),button);
-		gtk_widget_show (button);
+		gtk_menu_append(GTK_MENU (historymenu),button);
+		gtk_widget_show(button);
 
 		while (hist_ptr[0] != '/') hist_ptr--;
 		hist_ptr[0] = '\0';
 		}
 	g_free(hist);
 	/* looks like we have to add the root directory manually */
-	button = gtk_menu_item_new_with_label ("/");
+	button = gtk_menu_item_new_with_label("/");
 
-	gtk_signal_connect (GTK_OBJECT (button), "activate",
-		(GtkSignalFunc) history_menu_select, "/");
+	gtk_signal_connect(GTK_OBJECT(button), "activate",
+			   GTK_SIGNAL_FUNC(history_menu_select), path);
 
-	gtk_menu_append (GTK_MENU (historymenu), button);
-	gtk_widget_show (button);
+	gtk_menu_append(GTK_MENU (historymenu), button);
+	gtk_widget_show(button);
 
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(historypulldown), historymenu);
 	gtk_entry_set_text(GTK_ENTRY(pathentry), path);
@@ -1447,6 +1466,7 @@ void playlist_window_update_titles(void)
 		gtk_window_set_title (GTK_WINDOW (playlist_window), buf);
 		g_free(buf);
 		}
+	display_set_playlist(playlist_filename);
 }
 
 /*
@@ -1841,13 +1861,13 @@ static void playlist_window_dnd_init(void)
 	gtk_drag_source_set(dirlist_clist, GDK_BUTTON2_MASK,
 			filelist_drag_types, filelist_drag_count, GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 	gtk_signal_connect(GTK_OBJECT(dirlist_clist), "drag_data_get",
-			playlist_window_dirlist_set_dnd_data, NULL);
+			   GTK_SIGNAL_FUNC(playlist_window_dirlist_set_dnd_data), NULL);
 
 	/* file clist */
 	gtk_drag_source_set(filelist_clist, GDK_BUTTON2_MASK,
 			filelist_drag_types, filelist_drag_count, GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 	gtk_signal_connect(GTK_OBJECT(filelist_clist), "drag_data_get",
-			playlist_window_filelist_set_dnd_data, NULL);
+			   GTK_SIGNAL_FUNC(playlist_window_filelist_set_dnd_data), NULL);
 
 	/* playlist clist */
 	gtk_drag_dest_set(playlist_clist,
@@ -1855,16 +1875,16 @@ static void playlist_window_dnd_init(void)
 			playlist_drop_types, playlist_drop_count, 
 			GDK_ACTION_COPY);
 	gtk_signal_connect(GTK_OBJECT(playlist_clist), "drag_data_received",
-			playlist_window_playlist_get_dnd_data , NULL);
+			   GTK_SIGNAL_FUNC(playlist_window_playlist_get_dnd_data), NULL);
 	gtk_signal_connect(GTK_OBJECT(playlist_clist), "drag_motion",
-			playlist_window_dnd_motion, NULL);
+			   GTK_SIGNAL_FUNC(playlist_window_dnd_motion), NULL);
 	gtk_signal_connect(GTK_OBJECT(playlist_clist), "drag_leave",
-			playlist_window_dnd_leave, NULL);
+			   GTK_SIGNAL_FUNC(playlist_window_dnd_leave), NULL);
 
 	gtk_drag_source_set(playlist_clist, GDK_BUTTON2_MASK,
 			playlist_drag_types, playlist_drag_count, GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 	gtk_signal_connect(GTK_OBJECT(playlist_clist), "drag_data_get",
-			playlist_window_playlist_set_dnd_data, NULL);
+			   GTK_SIGNAL_FUNC(playlist_window_playlist_set_dnd_data), NULL);
 }
 
 /*
@@ -1965,7 +1985,8 @@ static void playlist_window_build(void)
 	/* set up playlist window */
 	playlist_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_policy(GTK_WINDOW(playlist_window), TRUE, TRUE, FALSE);
-	gtk_signal_connect (GTK_OBJECT (playlist_window), "delete_event",(GtkSignalFunc) playlist_window_hide, NULL);
+	gtk_signal_connect(GTK_OBJECT (playlist_window), "delete_event",
+			   GTK_SIGNAL_FUNC(playlist_window_hide), NULL);
 	buf = g_strconcat(playlist_filename,_(" - GQmpeg Playlist"), NULL);
 	gtk_window_set_title (GTK_WINDOW (playlist_window), buf);
 	g_free(buf);
@@ -2018,7 +2039,8 @@ static void playlist_window_build(void)
 	/* directory list */
 	dirlist_clist=gtk_clist_new_with_titles (1, dir_title);
 	gtk_clist_set_column_auto_resize(GTK_CLIST (dirlist_clist), 0, TRUE);
-	gtk_signal_connect (GTK_OBJECT (dirlist_clist), "button_press_event",(GtkSignalFunc) dirlist_press_cb, NULL);
+	gtk_signal_connect(GTK_OBJECT(dirlist_clist), "button_press_event",
+			   GTK_SIGNAL_FUNC(dirlist_press_cb), NULL);
 #if 0
 	gtk_widget_set_usize (dirlist_clist, list_width, DIR_DEFAULT_HEIGHT);
 #endif
@@ -2037,7 +2059,8 @@ static void playlist_window_build(void)
 	filelist_clist=gtk_clist_new_with_titles (1, file_title);
 	gtk_clist_set_column_auto_resize(GTK_CLIST (filelist_clist), 0, TRUE);
 	gtk_clist_set_selection_mode(GTK_CLIST(filelist_clist), GTK_SELECTION_EXTENDED);
-	gtk_signal_connect (GTK_OBJECT (filelist_clist), "button_press_event",(GtkSignalFunc) filelist_press_cb, NULL);
+	gtk_signal_connect(GTK_OBJECT(filelist_clist), "button_press_event",
+			   GTK_SIGNAL_FUNC(filelist_press_cb), NULL);
 #if 0
 	gtk_widget_set_usize (filelist_clist, list_width, -1);
 #endif
@@ -2062,150 +2085,163 @@ static void playlist_window_build(void)
 	gtk_widget_show(hbox1);
 
 	button = gtk_button_new_with_label(_("Home"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) change_dir_to_home, NULL);
-	gtk_box_pack_start (GTK_BOX (hbox1), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(change_dir_to_home), NULL);
+	gtk_box_pack_start(GTK_BOX(hbox1), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("Close"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) playlist_window_hide, NULL);
-	gtk_box_pack_start (GTK_BOX (hbox1), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(playlist_window_hide), NULL);
+	gtk_box_pack_start(GTK_BOX(hbox1), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
 	frame = gtk_frame_new(_("Playlist"));
 	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 0);
 	gtk_widget_show(frame);
 
-	vbox2 = gtk_vbox_new (FALSE, 0);
-	gtk_container_border_width (GTK_CONTAINER (vbox2), 5);
-	gtk_container_add (GTK_CONTAINER (frame),vbox2);
-	gtk_widget_show (vbox2);
+	vbox2 = gtk_vbox_new(FALSE, 0);
+	gtk_container_border_width(GTK_CONTAINER (vbox2), 5);
+	gtk_container_add(GTK_CONTAINER(frame),vbox2);
+	gtk_widget_show(vbox2);
 
 	button = gtk_button_new_with_label(_("Save"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) save_pressed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(save_pressed), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("Save as..."));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) save_as_pressed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(save_as_pressed), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
 	button = gtk_check_button_new_with_label(_("save mode"));
 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), save_mode_in_playlist);
-	gtk_signal_connect (GTK_OBJECT (button), "toggled",(GtkSignalFunc) save_mode_changed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "toggled",
+			   GTK_SIGNAL_FUNC(save_mode_changed), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("Open"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) btn_load_playlist_pressed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(btn_load_playlist_pressed), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("Append"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) append_pressed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(append_pressed), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
-#if 0
-	button = gtk_button_new_with_label(_("Import"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) import_pressed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
-	gtk_widget_show(button);
-#endif
-
 	frame = gtk_frame_new(_("Move Song"));
 	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 0);
 	gtk_widget_show(frame);
 
-	hbox1 = gtk_hbox_new (FALSE, 0);
-	gtk_container_border_width (GTK_CONTAINER (hbox1), 5);
-	gtk_container_add (GTK_CONTAINER (frame),hbox1);
+	hbox1 = gtk_hbox_new(FALSE, 0);
+	gtk_container_border_width(GTK_CONTAINER(hbox1), 5);
+	gtk_container_add(GTK_CONTAINER(frame),hbox1);
 	gtk_widget_show(hbox1);
 
-	vbox2 = gtk_vbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox1), vbox2, TRUE, TRUE, 0);
-	gtk_widget_show (vbox2);
+	vbox2 = gtk_vbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox1), vbox2, TRUE, TRUE, 0);
+	gtk_widget_show(vbox2);
 
 	button = gtk_button_new_with_label(_("Top"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) top_button_pressed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(top_button_pressed), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("End"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) end_button_pressed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(end_button_pressed), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
-	vbox2 = gtk_vbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox1), vbox2, TRUE, TRUE, 0);
-	gtk_widget_show (vbox2);
+	vbox2 = gtk_vbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox1), vbox2, TRUE, TRUE, 0);
+	gtk_widget_show(vbox2);
 
 	button = gtk_button_new_with_label(_("Up"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) up_button_pressed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(up_button_pressed), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("Down"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) down_button_pressed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(down_button_pressed), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
 	frame = gtk_frame_new(_("Songs"));
 	gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 0);
 	gtk_widget_show(frame);
 
-	vbox2 = gtk_vbox_new (FALSE, 0);
-	gtk_container_border_width (GTK_CONTAINER (vbox2), 5);
-	gtk_container_add (GTK_CONTAINER (frame),vbox2);
-	gtk_widget_show (vbox2);
+	vbox2 = gtk_vbox_new(FALSE, 0);
+	gtk_container_border_width(GTK_CONTAINER(vbox2), 5);
+	gtk_container_add(GTK_CONTAINER(frame),vbox2);
+	gtk_widget_show(vbox2);
 
 	button = gtk_button_new_with_label(_("Add All"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) add_all_button_pressed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(add_all_button_pressed), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("Add"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) plw_filelist_add_pressed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(plw_filelist_add_pressed), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("Insert"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) plw_filelist_insert_pressed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(plw_filelist_insert_pressed), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("Remove"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) plw_playlist_remove_pressed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(plw_playlist_remove_pressed), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("Remove All"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) remove_all_button_pressed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(remove_all_button_pressed), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
 	button = gtk_hseparator_new();
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 5);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("Add Custom"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",(GtkSignalFunc) add_custom_pressed, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox2), button, TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(add_custom_pressed), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox2), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
 
 	scrolled = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
-				GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
+				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
 	gtk_paned_add2(GTK_PANED(paned), scrolled);
 	gtk_widget_show(scrolled);
 
 	/* play list */
 	playlist_clist_construct_titles(play_title);
-	playlist_clist=gtk_clist_new_with_titles(playlist_view_elements, play_title);
+	playlist_clist = gtk_clist_new_with_titles(playlist_view_elements, play_title);
 	gtk_clist_set_selection_mode(GTK_CLIST(playlist_clist), GTK_SELECTION_EXTENDED);
 
-	gtk_signal_connect (GTK_OBJECT (playlist_clist), "button_press_event",(GtkSignalFunc) playlist_press_cb, NULL);
-	gtk_widget_set_usize (playlist_clist, list_width * 2, -1);
+	gtk_signal_connect(GTK_OBJECT(playlist_clist), "button_press_event",
+			   GTK_SIGNAL_FUNC(playlist_press_cb), NULL);
+	gtk_signal_connect(GTK_OBJECT(playlist_clist), "select_row",
+			   GTK_SIGNAL_FUNC(playlist_select_row_cb), NULL);
+	gtk_widget_set_usize(playlist_clist, list_width * 2, -1);
 
 	gtk_clist_set_column_width(GTK_CLIST(playlist_clist), 0, playlist_view_widths[0]);
 
