@@ -87,7 +87,7 @@ static WidgetData *parse_button(gchar **vector, gint length, const gchar *dirpat
 		clip_filename = NULL;
 		}
 
-	button = button_new_from_file(filename, prelight, light, x, y, clip_filename);
+	button = button_new_from_file(filename, x, y, prelight, light, clip_filename);
 
 	if (button)
 		{
@@ -233,7 +233,7 @@ static void parse_number(gchar **vector, gint length, const gchar *dirpath,
 			}
 		}
 
-	number = number_new(digit, num_length, zeros, center, x , y);
+	number = number_new(digit, x, y, num_length, zeros, center);
 	digit_unref(digit);
 
 	if (number)
@@ -278,7 +278,7 @@ static void parse_item(gchar **vector, gint length, gchar const *dirpath,
 	y = parse_to_number(vector[p]);
 	p++;
 
-	item = item_new_from_file(filename, sections, x , y);
+	item = item_new_from_file(filename, x, y, sections);
 
 	if (item)
 		{
@@ -330,7 +330,7 @@ static void parse_text(gchar **vector, gint length, const gchar *dirpath,
 	p++;
 
 
-	text = text_new_from_file(filename, text_length, extended, FALSE, x, y);
+	text = text_new_from_file(filename, x, y, text_length, FALSE, extended);
 
 	if (text)
 		{
@@ -389,8 +389,8 @@ static void parse_slider(gchar **vector, gint length, const gchar *dirpath,
 	y = parse_to_number(vector[p]);
 	p++;
 
-	slider = slider_new_from_file(filename, TRUE, prelight, vertical, reversed,
-				      slider_length, x, y, item_key);
+	slider = slider_new_from_file(filename, x, y, slider_length,
+				      vertical, reversed, TRUE, prelight, item_key);
 
 	if (slider)
 		{
@@ -464,10 +464,11 @@ static void parse_dial(gchar **vector, gint length, const gchar *dirpath,
 		clip_filename = NULL;
 		}
 
-	dial = dial_new_from_file(filename, has_press, has_prelight, reversed,
-				  angle_start, angle_end, offset_x, offset_y,
+	dial = dial_new_from_file(filename, x, y,
 				  center_x, center_y,
-				  x, y, w, h, clip_filename, NULL);
+				  angle_start, angle_end, offset_x, offset_y,
+				  has_press, has_prelight, reversed,
+				  w, h, clip_filename, NULL);
 
 	if (dial)
 		{
