@@ -72,7 +72,7 @@ static TextData *wamp_text(gchar *path, gint l, gint x, gint y, SkinData *skin)
 	gdk_pixbuf_unref(pb_temp);
 
 	font = font_new(pb, FALSE);
-	text = text_new(font, l, FALSE, x, y);
+	text = text_new(font, x, y, l, FALSE);
 	font_unref(font);
 	text_register(skin, text, "title", NULL);
 
@@ -150,7 +150,7 @@ static WidgetData *wamp_button(GdkPixbuf *pb_temp, gint lit, gint x, gint y, gin
 		pixbuf_copy_area(pb_temp, ox, oy + (ly * 3), pb, lx * 3, 0, lx, ly, TRUE);
 		}
 
-	button = button_new(pb, FALSE, lit, x, y, NULL);
+	button = button_new(pb, x, y, FALSE, lit, NULL);
 	return button_register(skin, button, key, NULL);
 }
 
@@ -171,7 +171,7 @@ static WidgetData *wamp_button_v(GdkPixbuf *pb_temp, gint lit, gint x, gint y, g
 		pixbuf_copy_area(pb_temp, ox, oy + (ly * 3), pb, lx * 3, 0, lx, ly, TRUE);
 		}
 
-	button = button_new(pb, FALSE, lit, x, y, NULL);
+	button = button_new(pb, x, y, FALSE, lit, NULL);
 	return button_register(skin, button, key, NULL);
 }
 
@@ -210,7 +210,7 @@ static void wamp_item_monoster(gchar *path, SkinData *skin)
 
 	gdk_pixbuf_unref(pb_temp);
 
-	item = item_new(pb, 3, 212, 41);	
+	item = item_new(pb, 212, 41, 3);	
 	item_register(skin, item, "stereo", NULL);
 }
 
@@ -231,7 +231,7 @@ static void wamp_item_status(gchar *path, SkinData *skin)
 
 	gdk_pixbuf_unref(pb_temp);
 
-	item = item_new(pb, 3, 26, 28);
+	item = item_new(pb, 26, 28, 3);
 	item_register(skin, item, "status", NULL);
 }
 
@@ -248,7 +248,7 @@ static void wamp_button_alt(GdkPixbuf *pb_temp, gint x, gint y, gint ox, gint oy
 	pixbuf_copy_area(pb_temp, ox + lx * 2, oy, pb, lx * 2, 0, lx, ly, TRUE);
 	pixbuf_copy_area(pb_temp, ox + lx * 2, oy + ly, pb, lx * 3, 0, lx, ly, TRUE);
 
-	button = button_new(pb, FALSE, TRUE, x, y, NULL);
+	button = button_new(pb, x, y, FALSE, TRUE, NULL);
 	button_register(skin, button, key, NULL);
 }
 
@@ -263,7 +263,7 @@ static void wamp_button_small(GdkPixbuf *pb_temp, gint x, gint y, gint ox, gint
 	pixbuf_copy_area(pb_temp, ox, oy, pb, 0, 0, 10, 10, TRUE);
 	pixbuf_copy_area(pb_temp, ox, oy, pb, 10, 0, 10, 10, TRUE);
 
-	button = button_new(pb, FALSE, FALSE, x, y, NULL);
+	button = button_new(pb, x, y, FALSE, FALSE, NULL);
 	button_register(skin, button, key, NULL);
 }
 
@@ -283,7 +283,7 @@ static void wamp_slider(gchar *path, gint prelight, gint length, gint x, gint y,
 
 	gdk_pixbuf_unref(pb_temp);
 
-	slider = slider_new(pb, TRUE, prelight, FALSE, FALSE, length, x, y, NULL);
+	slider = slider_new(pb, x, y, length, FALSE, FALSE, TRUE, prelight, NULL);
 	slider_register(skin, slider, "position", NULL);
 }
 
@@ -304,7 +304,7 @@ static void wamp_vol_slider(gchar *path, gint prelight, gint length, gint x, gin
 
 	gdk_pixbuf_unref(pb_temp);
 
-	slider = slider_new(pb, TRUE, prelight, FALSE, FALSE, length, x, y, "wamp_volume");
+	slider = slider_new(pb, x, y, length, FALSE, FALSE, TRUE, prelight, "wamp_volume");
 	slider_register(skin, slider, "volume", NULL);
 }
 
@@ -325,7 +325,7 @@ static void wamp_bal_slider(gchar *path, gint prelight, gint length, gint x, gin
 
 	gdk_pixbuf_unref(pb_temp);
 
-	slider = slider_new(pb, TRUE, prelight, FALSE, FALSE, length, x, y, "wamp_balance");
+	slider = slider_new(pb, x, y, length, FALSE, FALSE, TRUE, prelight, "wamp_balance");
 	slider_register(skin, slider, "balance", NULL);
 }
 
@@ -348,7 +348,7 @@ static void wamp_vol_item(gchar *path, SkinData *skin)
 
 	gdk_pixbuf_unref(pb_temp);
 
-	item = item_new(pb, 28, 107, 58);
+	item = item_new(pb, 107, 58, 28);
 	item_register(skin, item, "wamp_volume", NULL);
 }
 
@@ -375,7 +375,7 @@ static void wamp_bal_item(gchar *path, SkinData *skin)
 
 	gdk_pixbuf_unref(pb_temp);
 
-	item = item_new(pb, 28, 177, 58);
+	item = item_new(pb, 177, 58, 28);
 	item_register(skin, item, "wamp_balance", NULL);
 }
 
@@ -384,7 +384,7 @@ static void wamp_number(DigitData *digit, gint length, gint zeros, gint center,
 {
 	NumberData *number;
 
-	number = number_new(digit, length, zeros, center, x, y);
+	number = number_new(digit, x, y, length, zeros, center);
 	number_register(skin, number, key, NULL);
 }
 
