@@ -23,6 +23,8 @@
 #include "ui2_widget.h"
 #include "ui_pixbuf_ops.h"
 
+#include <gdk/gdkkeysyms.h> /* for key values */
+
 
 typedef struct _ButtonCallbackData ButtonCallbackData;
 struct _ButtonCallbackData
@@ -46,10 +48,13 @@ static WidgetType type_id = -1;
  *-----------------------------
  */
 
-ButtonData *button_new(GdkPixbuf *pb, gint prelight, gint light, gint x, gint y, GdkPixbuf *clip_pb)
+ButtonData *button_new(GdkPixbuf *pb, gint x, gint y,
+		       gint prelight, gint light, GdkPixbuf *clip_pb)
 {
 	ButtonData *button;
 
+	button_type_init();
+
 	util_size(&x);
 	util_size(&y);
 
@@ -106,7 +111,7 @@ ButtonData *button_new(GdkPixbuf *pb, gint prelight, gint light, gint x, gint y,
 			{
 			pixbuf_copy_channel(clip_pb, 0, 0,
 					   button->pixbuf, i * button->width, 0,
-					   button->width, button->height, 4);
+					   button->width, button->height, 3);
 			}
 
 		gdk_pixbuf_unref(clip_pb);
@@ -122,7 +127,8 @@ ButtonData *button_new(GdkPixbuf *pb, gint prelight, gint light, gint x, gint y,
 	return button;
 }
 
-ButtonData *button_new_from_data(gchar **data, gint prelight, gint light, gint x, gint y, gchar **clip_data)
+ButtonData *button_new_from_data(gchar **data, gint x, gint y,
+				 gint prelight, gint light, gchar **clip_data)
 {
 	GdkPixbuf *pb;
 	GdkPixbuf *clip_pb;
@@ -138,11 +144,12 @@ ButtonData *button_new_from_data(gchar **data, gint prelight, gint light, gint x
 		clip_pb = NULL;
 		}
 	
-	return button_new(pb, prelight, light, x, y, clip_pb);
+	return button_new(pb, x, y, prelight, light, clip_pb);
 }
 
 
-ButtonData *button_new_from_file(const gchar *file, gint prelight, gint light, gint x, gint y, const gchar *clip_file)
+ButtonData *button_new_from_file(const gchar *file, gint x, gint y,
+				 gint prelight, gint light, const gchar *clip_file)
 {
 	GdkPixbuf *pb;
 	GdkPixbuf *clip_pb;
@@ -158,7 +165,7 @@ ButtonData *button_new_from_file(const gchar *file, gint prelight, gint light, g
 		clip_pb = NULL;
 		}
 
-	return button_new(pb, prelight, light, x, y, clip_pb);
+	return button_new(pb, x, y, prelight, light, clip_pb);
 }
 
 void button_free(ButtonData *button)
@@ -236,7 +243,7 @@ static void button_draw_real(ButtonData *button, gint prelight, gint pressed, gi
 					 button->width, button->height, FALSE);
 			}
 
-		ui_display_render_area(ui, button->x, button->y, button->width, button->height);
+		ui_display_render_area(ui, button->x, button->y, button->width, button->height, button->wd);
 		}
 
 	if (!force)
@@ -383,6 +390,36 @@ static void button_release(gpointer data, const gchar *key, gint x, gint y, GdkP
 		}
 }
 
+static gint button_key_event_cb(gpointer data, const gchar *key,
+				GdkEventKey *event, GdkPixbuf *pb, UIData *ui)
+{
+	if (event->keyval == GDK_space)
+		{
+		ButtonData *button = data;
+		ButtonCallbackData *cd;
+
+		cd = ui_get_registered_callbacks(ui, key, type_id);
+
+		/* run the signal gamut, for compatibility */
+		if (cd && cd->button_press_func)
+			{
+			cd->button_press_func(button, key, cd->button_press_data);
+			}
+		if (cd && cd->button_release_func)
+			{
+			cd->button_release_func(button, key, cd->button_release_data);
+			}
+		if (cd && cd->button_click_func)
+			{
+			cd->button_click_func(button, key, cd->button_click_data);
+			}
+		
+		return TRUE;
+		}
+
+	return FALSE;
+}
+
 static void button_back_set(gpointer data, GdkPixbuf *pb)
 {
 	ButtonData *button = data;
@@ -441,7 +478,7 @@ static WidgetData *button_parse(SkinData *skin, GList *list, const gchar *skin_d
 	light = key_list_read_bool(list, "indicator");
 	clipname = key_list_read_path(list, "clip_mask", skin_dir);
 
-	button = button_new_from_file(filename, prelight, light, x, y, clipname);
+	button = button_new_from_file(filename, x, y, prelight, light, clipname);
 
 	if (button)
 		{
@@ -468,7 +505,22 @@ static WidgetData *button_parse(SkinData *skin, GList *list, const gchar *skin_d
 
 WidgetData *button_register(SkinData *skin, ButtonData *button, const gchar *key, const gchar *text_id)
 {
-	return skin_register_widget(skin, key, text_id, type_id, button);
+	WidgetData *wd;
+
+	wd = skin_register_widget(skin, key, text_id, type_id, button);
+	button->wd = wd;
+
+	return wd;
+}
+
+WidgetData *button_register_to_skin(const gchar *key, gchar **data, gint x, gint y,
+				    gint prelight, gint light, gchar **clip_data,
+				    SkinData *skin, const gchar *text_id)
+{
+	ButtonData *button;
+
+	button = button_new_from_data(data, x, y, prelight, light, clip_data);
+	return button_register(skin, button, key, text_id);
 }
 
 RegisterData *button_register_key(const gchar *key, UIData *ui,
@@ -479,6 +531,8 @@ RegisterData *button_register_key(const gchar *key, UIData *ui,
 {
 	ButtonCallbackData *cd;
 
+	button_type_init();
+
 	cd = g_new0(ButtonCallbackData, 1);
 
 	cd->status_get_func = status_get_func;
@@ -557,6 +611,8 @@ void button_type_init(void)
 	od->func_back = button_back_set;
 	od->func_free = button_free_cb;
 
+	od->func_focus_key_event = button_key_event_cb;
+
 	od->func_get_geometry = button_get_geometry;
 	od->func_set_coord = button_set_coord;
 
@@ -566,3 +622,5 @@ void button_type_init(void)
 }
 
 
+
+
