@@ -200,8 +200,9 @@ static void button_edit_add_cb(GtkWidget *widget, gpointer data)
 		return;
 		}
 
-	button = button_new_from_file(image, ui_edit_toggle_get(pd->has_prelight_button),
-				      ui_edit_toggle_get(pd->has_light_button), 0, 0, clip_mask);
+	button = button_new_from_file(image, 0, 0,
+				      ui_edit_toggle_get(pd->has_prelight_button),
+				      ui_edit_toggle_get(pd->has_light_button), clip_mask);
 	if (button)
 		{
 		WidgetData *wd;
@@ -281,11 +282,13 @@ static GtkWidget *button_edit_page_new(EditData *ed)
 
 	button = gtk_button_new_with_label(_("Add"));
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", button_edit_add_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(button_edit_add_cb), pd);
 	gtk_widget_show(button);
 
 	pd->clist = ui_edit_clist_new(hbox, titles, 4);
-	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row", button_edit_page_clist_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row",
+			   GTK_SIGNAL_FUNC(button_edit_page_clist_cb), pd);
 
 	gtk_widget_show(hbox);
 
