@@ -56,10 +56,11 @@ static void real_dial_idle_cancel(DialData *dial);
  *-----------------------------
  */
 
-DialData *dial_new(GdkPixbuf *pb, gint has_press, gint has_prelight, gint reversed,
-		   gint angle_start, gint angle_end, gint offset_x, gint offset_y,
+DialData *dial_new(GdkPixbuf *pb, gint x, gint y,
 		   gint center_x, gint center_y,
-		   gint x, gint y, gint clip_w, gint clip_h, GdkPixbuf *clip_pb,
+		   gint angle_start, gint angle_end, gint offset_x, gint offset_y,
+		   gint has_press, gint has_prelight, gint reversed,
+		   gint clip_w, gint clip_h, GdkPixbuf *clip_pb,
 		   const gchar *item_key)
 {
 	DialData *dial;
@@ -68,6 +69,8 @@ DialData *dial_new(GdkPixbuf *pb, gint has_press, gint has_prelight, gint revers
 	gint max_radius;
 	gint t;
 
+	dial_type_init();
+
 	if (!pb)
 		{
 		if (clip_pb) gdk_pixbuf_unref(clip_pb);
@@ -190,10 +193,11 @@ DialData *dial_new(GdkPixbuf *pb, gint has_press, gint has_prelight, gint revers
 	return dial;
 }
 
-DialData *dial_new_from_data(gchar **data, gint has_press, gint has_prelight, gint reversed,
-			     gint angle_start, gint angle_end, gint offset_x, gint offset_y,
+DialData *dial_new_from_data(gchar **data, gint x, gint y,
 			     gint center_x, gint center_y,
-			     gint x, gint y, gint clip_w, gint clip_h, gchar **clip_data,
+			     gint angle_start, gint angle_end, gint offset_x, gint offset_y,
+			     gint has_press, gint has_prelight, gint reversed,
+			     gint clip_w, gint clip_h, gchar **clip_data,
 			     const gchar *item_key)
 {
 	GdkPixbuf *pb;
@@ -210,16 +214,18 @@ DialData *dial_new_from_data(gchar **data, gint has_press, gint has_prelight, gi
 		clip_pb = NULL;
 		}
 
-	return dial_new(pb, has_press, has_prelight, reversed,
-			angle_start, angle_end, offset_x, offset_y,
+	return dial_new(pb, x, y,
 			center_x, center_y,
-			x, y, clip_w, clip_h, clip_pb, item_key);
+			angle_start, angle_end, offset_x, offset_y,
+			has_press, has_prelight, reversed,
+			clip_w, clip_h, clip_pb, item_key);
 }
 
-DialData *dial_new_from_file(const gchar *file, gint has_press, gint has_prelight, gint reversed,
-			     gint angle_start, gint angle_end, gint offset_x, gint offset_y,
+DialData *dial_new_from_file(const gchar *file, gint x, gint y,
 			     gint center_x, gint center_y,
-			     gint x, gint y, gint clip_w, gint clip_h, const gchar *clip_file,
+			     gint angle_start, gint angle_end, gint offset_x, gint offset_y,
+			     gint has_press, gint has_prelight, gint reversed,
+			     gint clip_w, gint clip_h, const gchar *clip_file,
 			     const gchar *item_key)
 {
 	GdkPixbuf *pb;
@@ -236,10 +242,11 @@ DialData *dial_new_from_file(const gchar *file, gint has_press, gint has_preligh
 		clip_pb = NULL;
 		}
 
-	return dial_new(pb, has_press, has_prelight, reversed,
-			angle_start, angle_end, offset_x, offset_y,
+	return dial_new(pb, x, y,
 			center_x, center_y,
-			x, y, clip_w, clip_h, clip_pb, item_key);
+			angle_start, angle_end, offset_x, offset_y,
+			has_press, has_prelight, reversed,
+			clip_w, clip_h, clip_pb, item_key);
 }
 
 void dial_free(DialData *dial)
@@ -303,8 +310,8 @@ static void real_dial_render(DialData *dial, GdkPixbuf *pb, UIData *ui)
 
 	if (dial->item_key)
 		{
-		ui_widget_for_each_key(ui, dial->item_key, item_type_id(),
-				       dial_force_item_draw_cb, dial);
+		ui_widget_for_each_key_one(ui, dial->item_key, item_type_id(),
+					   dial_force_item_draw_cb, dial);
 		}
 
 	handle = dial_get_handle_pixbuf(dial, dial->pushed, dial->prelit);
@@ -315,7 +322,7 @@ static void real_dial_render(DialData *dial, GdkPixbuf *pb, UIData *ui)
 					       dial->x, dial->y, dial->width, dial->height,
 					       dial->clip_mask ? dial->pixbuf : NULL);
 
-	ui_display_render_area(ui, dial->x, dial->y, dial->width, dial->height);
+	ui_display_render_area(ui, dial->x, dial->y, dial->width, dial->height, NULL);
 }
 
 static gint real_dial_idle_cb(gpointer data)
@@ -728,10 +735,11 @@ static WidgetData *dial_parse(SkinData *skin, GList *list, const gchar *skin_dir
 	reverse = key_list_read_bool(list, "reversed");
 	item_key = key_list_read_chars(list, "item_link", NULL);
 
-	dial = dial_new_from_file(filename, has_press, prelight, reverse,
-				  angle_begin, angle_end, offset_x, offset_y,
+	dial = dial_new_from_file(filename, x, y,
 				  center_x, center_y,
-				  x, y, clip_w, clip_h, clip_file, item_key);
+				  angle_begin, angle_end, offset_x, offset_y,
+				  has_press, prelight, reverse,
+				  clip_w, clip_h, clip_file, item_key);
 	if (dial)
 		{
 		wd = dial_register(skin, dial, key, NULL);
@@ -762,6 +770,23 @@ WidgetData *dial_register(SkinData *skin, DialData *dial, const gchar *key, cons
         return skin_register_widget(skin, key, text_id, type_id, dial);
 }
 
+WidgetData *dial_register_to_skin(const gchar *key, gchar **data, gint x, gint y,
+				  gint center_x, gint center_y,
+				  gint angle_start, gint angle_end, gint offset_x, gint offset_y,
+				  gint has_press, gint has_prelight, gint reversed,
+				  gint clip_w, gint clip_h, gchar **clip_data,
+				  const gchar *item_key,
+				  SkinData *skin, const gchar *text_id)
+{
+	DialData *dial;
+
+	dial = dial_new_from_data(data, x, y, center_x, center_y,
+				  angle_start, angle_end, offset_x, offset_y,
+				  has_press, has_prelight, reversed,
+				  clip_w, clip_h, clip_data, item_key);
+	return dial_register(skin, dial, key, text_id);
+}
+
 RegisterData *dial_register_key(const gchar *key, UIData *ui,
 				gfloat (*status_get_func)(DialData *dial, const gchar *key, gpointer data), gpointer status_get_data,
 				void (*dial_press_func)(DialData *dial, const gchar *key, gfloat value, gpointer data), gpointer dial_press_data,
@@ -770,6 +795,8 @@ RegisterData *dial_register_key(const gchar *key, UIData *ui,
 {
 	DialCallbackData *cd;
 
+	dial_type_init();
+
 	cd = g_new0(DialCallbackData, 1);
 
 	cd->status_get_func = status_get_func;
@@ -863,5 +890,3 @@ void dial_type_init(void)
 	dial_type_init_edit(od);
 }
 
-
-
