@@ -57,24 +57,35 @@ struct _DialData
 };
 
 
-DialData *dial_new(GdkPixbuf *pb, gint has_press, gint has_prelight, gint reversed,
-		   gint angle_start, gint angle_end, gint offset_x, gint offset_y,
+DialData *dial_new(GdkPixbuf *pb, gint x, gint y,
 		   gint center_x, gint center_y,
-		   gint x, gint y, gint clip_w, gint clip_h, GdkPixbuf *clip_pb,
+		   gint angle_start, gint angle_end, gint offset_x, gint offset_y,
+		   gint has_press, gint has_prelight, gint reversed,
+		   gint clip_w, gint clip_h, GdkPixbuf *clip_pb,
 		   const gchar *item_key);
-DialData *dial_new_from_data(gchar **data, gint has_press, gint has_prelight, gint reversed,
-			     gint angle_start, gint angle_end, gint offset_x, gint offset_y,
+DialData *dial_new_from_data(gchar **data, gint x, gint y,
 			     gint center_x, gint center_y,
-			     gint x, gint y, gint clip_w, gint clip_h, gchar **clip_data,
+			     gint angle_start, gint angle_end, gint offset_x, gint offset_y,
+			     gint has_press, gint has_prelight, gint reversed,
+			     gint clip_w, gint clip_h, gchar **clip_data,
 			     const gchar *item_key);
-DialData *dial_new_from_file(const gchar *file, gint has_press, gint has_prelight, gint reversed,
-			     gint angle_start, gint angle_end, gint offset_x, gint offset_y,
+DialData *dial_new_from_file(const gchar *file, gint x, gint y,
 			     gint center_x, gint center_y,
-			     gint x, gint y, gint clip_w, gint clip_h, const gchar *clip_file,
+			     gint angle_start, gint angle_end, gint offset_x, gint offset_y,
+			     gint has_press, gint has_prelight, gint reversed,
+			     gint clip_w, gint clip_h, const gchar *clip_file,
 			     const gchar *item_key);
 void dial_free(DialData *dial);
 
 WidgetData *dial_register(SkinData *skin, DialData *dial, const gchar *key, const gchar *text_id);
+WidgetData *dial_register_to_skin(const gchar *key, gchar **data, gint x, gint y,
+				  gint center_x, gint center_y,
+				  gint angle_start, gint angle_end, gint offset_x, gint offset_y,
+				  gint has_press, gint has_prelight, gint reversed,
+				  gint clip_w, gint clip_h, gchar **clip_data,
+				  const gchar *item_key,
+				  SkinData *skin, const gchar *text_id);
+
 RegisterData *dial_register_key(const gchar *key, UIData *ui,
 				gfloat (*status_get_func)(DialData *dial, const gchar *key, gpointer data), gpointer status_get_data,
 				void (*dial_press_func)(DialData *dial, const gchar *key, gfloat value, gpointer data), gpointer dial_press_data,
