@@ -228,7 +228,7 @@ static void dial_edit_props_reversed_cb(GtkWidget *button, gpointer data)
 static void dial_edit_props_item_key_cb(GtkWidget *entry, gpointer data)
 {
 	DialDetail *dd = data;
-	gchar *buf;
+	const gchar *buf;
 
 	g_free(dd->dial->item_key);
 	buf = gtk_entry_get_text(GTK_ENTRY(entry));
@@ -359,44 +359,54 @@ static gpointer dial_edit_props(UIData *ui, WidgetData *wd, GtkWidget *vbox, gpo
 		hbox2 = ui_edit_frame_new(hbox, TRUE, _("Rotation axis"));
 
 		dd->spin_center_x = ui_edit_spin_new(hbox2, _("x:"), -1000, 1000, &adj);
-		gtk_signal_connect(adj, "value_changed", dial_edit_props_spin_cb, dd);
+		gtk_signal_connect(adj, "value_changed",
+				   GTK_SIGNAL_FUNC(dial_edit_props_spin_cb), dd);
 
 		dd->spin_center_y = ui_edit_spin_new(hbox2, _("y:"), -1000, 1000, &adj);
-		gtk_signal_connect(adj, "value_changed", dial_edit_props_spin_cb, dd);
+		gtk_signal_connect(adj, "value_changed",
+				   GTK_SIGNAL_FUNC(dial_edit_props_spin_cb), dd);
 
 		hbox2 = ui_edit_frame_new(hbox, TRUE, _("Handle rotation axis"));
 
 		dd->spin_handle_x = ui_edit_spin_new(hbox2, _("x:"), -1000, 1000, &adj);
-		gtk_signal_connect(adj, "value_changed", dial_edit_props_spin_cb, dd);
+		gtk_signal_connect(adj, "value_changed",
+				   GTK_SIGNAL_FUNC(dial_edit_props_spin_cb), dd);
 
 		dd->spin_handle_y = ui_edit_spin_new(hbox2, _("y:"), -1000, 1000, &adj);
-		gtk_signal_connect(adj, "value_changed", dial_edit_props_spin_cb, dd);
+		gtk_signal_connect(adj, "value_changed",
+				   GTK_SIGNAL_FUNC(dial_edit_props_spin_cb), dd);
 
 		hbox = ui_edit_frame_new(vbox, TRUE, _("Endpoints"));
 
 		dd->spin_begin = ui_edit_spin_new(hbox, _("begin:"), 0, 359, &adj);
 		gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(dd->spin_begin), TRUE);
-		gtk_signal_connect(adj, "value_changed", dial_edit_props_spin_cb, dd);
+		gtk_signal_connect(adj, "value_changed",
+				   GTK_SIGNAL_FUNC(dial_edit_props_spin_cb), dd);
 
 		dd->spin_end = ui_edit_spin_new(hbox, _("end:"), 0, 359, &adj);
 		gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(dd->spin_end), TRUE);
-		gtk_signal_connect(adj, "value_changed", dial_edit_props_spin_cb, dd);
+		gtk_signal_connect(adj, "value_changed",
+				   GTK_SIGNAL_FUNC(dial_edit_props_spin_cb), dd);
 
 		dd->reverse_button = ui_edit_toggle_new(hbox, _("Reverse"));
-		gtk_signal_connect(GTK_OBJECT(dd->reverse_button), "clicked", dial_edit_props_reversed_cb, dd);
+		gtk_signal_connect(GTK_OBJECT(dd->reverse_button), "clicked",
+				   GTK_SIGNAL_FUNC(dial_edit_props_reversed_cb), dd);
 
 		hbox = ui_edit_frame_new(vbox, TRUE, _("Clipping area"));
 
 		dd->spin_clip_w = ui_edit_spin_new(hbox, _("width:"), 1, 2000, &adj);
-		gtk_signal_connect(adj, "value_changed", dial_edit_props_spin_cb, dd);
+		gtk_signal_connect(adj, "value_changed",
+				   GTK_SIGNAL_FUNC(dial_edit_props_spin_cb), dd);
 
 		dd->spin_clip_h = ui_edit_spin_new(hbox, _("height:"), 1, 2000, &adj);
-		gtk_signal_connect(adj, "value_changed", dial_edit_props_spin_cb, dd);
+		gtk_signal_connect(adj, "value_changed",
+				   GTK_SIGNAL_FUNC(dial_edit_props_spin_cb), dd);
 
 		hbox = ui_edit_frame_new(vbox, FALSE, NULL);
 
 		dd->item_key_entry = ui_edit_entry_new(hbox, _("Item link key:"));
-		gtk_signal_connect(GTK_OBJECT(dd->item_key_entry), "changed", dial_edit_props_item_key_cb, dd);
+		gtk_signal_connect(GTK_OBJECT(dd->item_key_entry), "changed",
+				   GTK_SIGNAL_FUNC(dial_edit_props_item_key_cb), dd);
 		}
 
 	dd->dial = dial;
@@ -448,17 +458,16 @@ static void dial_edit_page_add_cb(GtkWidget *widget, gpointer data)
 		return;
 		}
 
-	dial = dial_new_from_file(image, ui_edit_toggle_get(pd->has_press_button),
-				  ui_edit_toggle_get(pd->has_prelight_button),
-				  ui_edit_toggle_get(pd->reverse_button),
+	dial = dial_new_from_file(image, 0, 0,
+				  ui_edit_spin_get(pd->center_x_spin),
+				  ui_edit_spin_get(pd->center_y_spin),
 				  ui_edit_spin_get(pd->angle_start_spin),
 				  ui_edit_spin_get(pd->angle_end_spin),
 				  ui_edit_spin_get(pd->offset_x_spin),
 				  ui_edit_spin_get(pd->offset_y_spin),
-				  ui_edit_spin_get(pd->center_x_spin),
-				  ui_edit_spin_get(pd->center_y_spin),
-				  0,
-				  0,
+				  ui_edit_toggle_get(pd->has_press_button),
+				  ui_edit_toggle_get(pd->has_prelight_button),
+				  ui_edit_toggle_get(pd->reverse_button),
 				  ui_edit_spin_get(pd->clip_w_spin),
 				  ui_edit_spin_get(pd->clip_h_spin),
 				  clip_mask, item_key);
@@ -574,7 +583,8 @@ static GtkWidget *dial_edit_page_new(EditData *ed)
 	pd->clip_h_spin = ui_edit_spin_new(frame, _("height:"), 1, 2000, NULL);
 
 	pd->clip_mask_entry = ui_edit_path_entry_new(vbox, _("Clip mask:"), "SLIK_dial_clip_mask");
-	gtk_signal_connect(GTK_OBJECT(pd->clip_mask_entry), "changed", dial_edit_page_clip_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(pd->clip_mask_entry), "changed",
+			   GTK_SIGNAL_FUNC(dial_edit_page_clip_cb), pd);
 
 	frame = ui_edit_frame_new(vbox, TRUE, _("Handle rotation axis"));
 	pd->offset_x_spin = ui_edit_spin_new(frame, _("x:"), -2000, 2000, NULL);
@@ -588,11 +598,13 @@ static GtkWidget *dial_edit_page_new(EditData *ed)
 
 	button = gtk_button_new_with_label(_("Add"));
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", dial_edit_page_add_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(dial_edit_page_add_cb), pd);
 	gtk_widget_show(button);
 
 	pd->clist = ui_edit_clist_new(hbox, titles, 4);
-	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row", dial_edit_page_clist_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row",
+			   GTK_SIGNAL_FUNC(dial_edit_page_clist_cb), pd);
 
 	gtk_widget_show(hbox);
 
