@@ -583,7 +583,7 @@ static gint ui_display_key_press(GtkWidget *w, GdkEventKey *event, gpointer data
 	UIData *ui = data;
 	gint ret = FALSE;
 
-	if (!slik_focus_enable) return FALSE;
+	if (!ui->skin || !slik_focus_enable) return FALSE;
 	if (ui->active_widget) return FALSE;
 
 	if (ui->focus_widget &&
@@ -639,7 +639,7 @@ static gint ui_display_focus_in(GtkWidget *widget, GdkEventFocus *event, gpointe
 
 	GTK_WIDGET_SET_FLAGS(widget, GTK_HAS_FOCUS);
 
-	if (!slik_focus_enable) return FALSE;
+	if (!ui->skin || !slik_focus_enable) return FALSE;
 
 	if (!ui->focus_widget) ui->focus_widget = ui_display_focus_next(ui, NULL, TRUE);
 	if (ui->focus_widget)
@@ -661,7 +661,7 @@ static gint ui_display_focus_out(GtkWidget *widget, GdkEventFocus *event, gpoint
 
 	GTK_WIDGET_UNSET_FLAGS(widget, GTK_HAS_FOCUS);
 
-	if (!slik_focus_enable) return FALSE;
+	if (!ui->skin || !slik_focus_enable) return FALSE;
 
 	if (ui->focus_widget)
 		{
@@ -691,6 +691,8 @@ static gint ui_display_motion(GtkWidget *w, GdkEventMotion *event, gpointer data
 
 	if (debug_skin) printf("motion:%d x %d\n", x, y);
 
+	if (!ui->skin) return FALSE;
+
 	if (ui->active_widget)
 		{
 		ui_widget_motion(ui, ui->active_widget, x, y);
@@ -740,6 +742,8 @@ static gint ui_display_pressed(GtkWidget *w, GdkEventButton *event, gpointer dat
 
 	if (debug_mode) printf("pressed:%d x %d, %d\n", x, y, event->button);
 
+	if (!ui->skin) return FALSE;
+
 	/* edit uis have no window, and we only want button 1 to work on them */
 	if (event->button > 1 && !ui->window) return FALSE;
 
@@ -806,6 +810,7 @@ static gint ui_display_released(GtkWidget *w, GdkEventButton *event, gpointer da
 
 	if (debug_mode) printf("released:%d x %d, %d\n", x, y, event->button);
 
+	if (!ui->skin) return FALSE;
 	if (!ui->in_press && !ui->active_widget) return FALSE;
 
 	gtk_grab_remove(ui->display);
@@ -838,6 +843,8 @@ static void ui_display_leave(GtkWidget *widget, GdkEventCrossing *event, gpointe
 {
 	UIData *ui = data;
 
+	if (!ui->skin) return;
+
 	if (!ui->active_widget)
 		{
 		GList *work;
