@@ -13,8 +13,11 @@
 #define UI2_DISPLAY_H
 
 
-void ui_display_render_area(UIData *ui, gint x, gint y, gint w, gint h);
-void ui_display_redraw_area(UIData *ui, gint x, gint y, gint w, gint h);
+void ui_display_render_area(UIData *ui, gint x, gint y, gint w, gint h, WidgetData *wd);
+void ui_display_redraw_area(UIData *ui, gint x, gint y, gint w, gint h, gint update);
+
+void ui_display_draw_focus(UIData *ui, GdkPixbuf *pixbuf, gint x, gint y, gint w, gint h,
+			   gint clip_x, gint clip_y, gint clip_w, gint clip_h, GdkPixbuf *clip_pb);
 
 void ui_display_sync_shape(UIData *ui);
 void ui_display_sync(UIData *ui, gint update);
