@@ -35,7 +35,7 @@ GtkWidget *ui_edit_key_entry_new(GtkWidget *box, UIData *ui, WidgetType type);
 GtkWidget *ui_edit_entry_new(GtkWidget *box, const gchar *text);
 void ui_edit_entry_set(GtkWidget *entry, const gchar *text);
 void ui_edit_entry_set_blocking(GtkWidget *entry, const gchar *text, gpointer block_data);
-gchar *ui_edit_entry_get(GtkWidget *entry);
+const gchar *ui_edit_entry_get(GtkWidget *entry);
 
 GtkWidget *ui_edit_spin_new(GtkWidget *box, const gchar *text, gint lower, gint upper, GtkObject **adjustment);
 void ui_edit_spin_set(GtkWidget *spin, gint value);
@@ -66,6 +66,7 @@ void ui_edit_widget_id_list_set_page(GList *list, WidgetType type, GtkWidget *pa
 void ui_edit_widget_draw_highlight(UIData *ui, WidgetData *wd, gint enable);
 
 void ui_edit_set_help_path(EditData *ed, const gchar *path);
+void ui_edit_set_skindata_list(EditData *ed, GList *list);
 
 void ui_edit_close(EditData *ed);
 
