@@ -162,7 +162,7 @@ static void item_edit_page_add_cb(GtkWidget *widget, gpointer data)
 		return;
 		}
 
-	item = item_new_from_file(image, ui_edit_spin_get(pd->section_spin), 0, 0);
+	item = item_new_from_file(image, 0, 0, ui_edit_spin_get(pd->section_spin));
 	if (item)
 		{
 		WidgetData *wd;
@@ -235,11 +235,13 @@ static GtkWidget *item_edit_page_new(EditData *ed)
 
 	item = gtk_button_new_with_label(_("Add"));
 	gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(item), "clicked", item_edit_page_add_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(item), "clicked",
+			   GTK_SIGNAL_FUNC(item_edit_page_add_cb), pd);
 	gtk_widget_show(item);
 
 	pd->clist = ui_edit_clist_new(hbox, titles, 3);
-	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row", item_edit_page_clist_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row",
+			   GTK_SIGNAL_FUNC(item_edit_page_clist_cb), pd);
 
 	gtk_widget_show(hbox);
 
