@@ -63,6 +63,7 @@ struct _ListDetail
 
 	GtkWidget *column_count_spin;
 	GtkWidget *column_clist;
+	GtkWidget *columns_right_toggle;
 
 	WidgetData *wd;
 	UIData *ui;
@@ -109,6 +110,8 @@ struct _ListListData
 	gint *column_widths;		/* array of column widths */
 	ListColumnFlags *column_flags;	/* column attributes (justify, etc. */
 	gchar **column_keys;		/* column text keys */
+
+	gint columns_right_justify;
 };
 
 typedef struct _ListPageColumn ListPageColumn;
@@ -197,6 +200,7 @@ static void list_edit_write(FILE *f, WidgetData *wd, SkinData *skin, const gchar
 	ui_edit_write_key_bool(f, "sizeable", ld->sizeable);
 
 	ui_edit_write_key_int(f, "columns", ld->column_count);
+	ui_edit_write_key_bool(f, "columns_right_justify", ld->columns_right_justify);
 
 	ui_edit_write_key_int(f, "border_top", ld->border_top);
 	ui_edit_write_key_int(f, "border_right", ld->border_right);
@@ -308,6 +312,7 @@ static gpointer list_edit_read(UIData *ui, WidgetData *wd, GList *list)
 	ll->sizeable = ld->sizeable;
 
 	ll->column_count = ld->column_count;
+	ll->columns_right_justify = ld->columns_right_justify;
 
 	ll->column_widths = g_new0(gint, ll->column_count);
 	memcpy(ll->column_widths, ld->column_widths, sizeof(gint) * ll->column_count);
@@ -449,6 +454,7 @@ static void list_details_pack(ListDetail *dt, GtkWidget *vbox, gint add_page)
 	gtk_widget_show(dt->column_clist);
 
 	dt->flag_column_spin = ui_edit_spin_new(page, _("Flag column:"), 0, 19, NULL);
+	dt->columns_right_toggle = ui_edit_toggle_new(page, _("Right justify"));
 
 	label = gtk_label_new(_("Columns"));
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, label);
@@ -560,6 +566,7 @@ static void list_edit_props_sync(ListDetail *dt)
 
 	ui_edit_entry_set(dt->flag_image_entry, ui_widget_get_data(wd, "flag_image"));
 	ui_edit_spin_set(dt->flag_sections_spin, ld->flag_sections);
+	ui_edit_toggle_set(dt->columns_right_toggle, ld->columns_right_justify);
 
 	ui_edit_spin_set_blocking(dt->column_count_spin, ld->column_count, dt);
 	ui_edit_spin_set_blocking(dt->flag_column_spin, ld->flag_column, dt);
@@ -822,7 +829,7 @@ static gint list_props_column_width_cb(ClistEditData *ced,
 	return FALSE;
 }
 
-static void list_props_column_clist_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
+static gint list_props_column_clist_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
 	ListDetail *dt = data;
 	gint row = -1;
@@ -832,9 +839,9 @@ static void list_props_column_clist_cb(GtkWidget *widget, GdkEventButton *event,
 	gint n;
 
 	gtk_clist_get_selection_info(GTK_CLIST(dt->column_clist), event->x, event->y, &row, &column);
-	if (row == -1 || column == -1) return;
+	if (row == -1 || column == -1) return FALSE;
 
-	if (row >= dt->ld->column_count) return;
+	if (row >= dt->ld->column_count) return FALSE;
 	ld = dt->ld;
 
 	switch (column)
@@ -896,7 +903,10 @@ static void list_props_column_clist_cb(GtkWidget *widget, GdkEventButton *event,
 		{
 		list_props_clist_update(dt, row);
 		ui_edit_widget_resync(dt->ui, dt->wd, TRUE, 0, 0);
+		return TRUE;
 		}
+
+	return FALSE;
 }
 
 static void list_props_flag_column_cb(GtkAdjustment *adj, gpointer data)
@@ -913,11 +923,21 @@ static void list_props_flag_column_cb(GtkAdjustment *adj, gpointer data)
 	ui_edit_widget_resync(dt->ui, dt->wd, TRUE, 0, 0);
 }
 
+static void list_props_column_right_cb(GtkWidget *widget, gpointer data)
+{
+	ListDetail *dt = data;
+	
+	dt->ld->columns_right_justify = ui_edit_toggle_get(dt->columns_right_toggle);
+
+	dt->ld->force_sync = TRUE;
+	ui_edit_widget_resync(dt->ui, dt->wd, TRUE, 0, 0);
+}
+
 static void list_props_connect_spin(GtkWidget *spin, ListDetail *dt, GtkSignalFunc func)
 {
 	GtkAdjustment *adj;
 
-	adj =gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
+	adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
 	gtk_signal_connect(GTK_OBJECT(adj), "value_changed", func, dt);
 }
 
@@ -962,6 +982,9 @@ static void list_props_signals(ListDetail *dt)
 			   GTK_SIGNAL_FUNC(list_props_column_clist_cb), dt);
 
 	list_props_connect_spin(dt->flag_column_spin, dt, GTK_SIGNAL_FUNC(list_props_flag_column_cb));
+
+	gtk_signal_connect(GTK_OBJECT(dt->columns_right_toggle), "clicked",
+			   GTK_SIGNAL_FUNC(list_props_column_right_cb), dt);
 }
 
 static gpointer list_edit_props(UIData *ui, WidgetData *wd, GtkWidget *vbox, gpointer detail)
@@ -1047,6 +1070,8 @@ static void list_edit_page_add_cb(GtkWidget *widget, gpointer data)
 		list_image_text(ld, gdk_pixbuf_new_from_file(text_image),
 				ui_edit_toggle_get(dt->text_extended_button));
 
+		list_set_column_justify(ld, ui_edit_toggle_get(dt->columns_right_toggle));
+
 		pb = NULL;
 		if (flag_image) pb = gdk_pixbuf_new_from_file(flag_image);
 
@@ -1195,6 +1220,7 @@ static void list_edit_page_sync(ListPage *pd, ListListData *ll)
 
 	ui_edit_spin_set_blocking(dt->column_count_spin, ll->column_count, pd);
 	ui_edit_spin_set(dt->flag_column_spin, ll->flag_column);
+	ui_edit_toggle_set(dt->columns_right_toggle, ll->columns_right_justify);
 
 	/* the column clist */
 	list_edit_page_free_columns(pd);
@@ -1300,11 +1326,13 @@ static GtkWidget *list_edit_page_new(EditData *ed)
 
 	button = gtk_button_new_with_label(_("Add"));
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", list_edit_page_add_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(list_edit_page_add_cb), pd);
 	gtk_widget_show(button);
 
 	pd->clist = ui_edit_clist_new(hbox, titles, 3);
-	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row", list_edit_page_clist_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row",
+			   GTK_SIGNAL_FUNC(list_edit_page_clist_cb), pd);
 
 	pd->column_list = g_list_append(pd->column_list,
 		page_column_new("column_1", 10, UI_LIST_COLUMN_SIZE_FIXED));
@@ -1346,4 +1374,3 @@ void list_type_init_edit(WidgetObjectData *od)
 	od->func_edit_page_add = list_edit_page_add;
 }
 
-
