@@ -14,9 +14,7 @@
 #include "ui2_main.h"
 
 #include "ui2_button.h"
-#include "ui2_default.h"
 #include "ui2_display.h"
-#include "ui2_init.h"
 #include "ui2_parse.h"
 #include "ui2_skin.h"
 #include "ui2_util.h"
@@ -39,8 +37,14 @@
 gint slik_smart_placement = TRUE;
 gint slik_remember_position = FALSE;
 
+gint slik_focus_enable = TRUE;
+
 gint slik_double_size = FALSE;
 
+gint slik_scale = FALSE;
+gfloat slik_scale_value = 1.0;
+gint slik_allow_shapes = TRUE;
+
 gint slik_colorshift_r = 0;
 gint slik_colorshift_g = 0;
 gint slik_colorshift_b = 0;
@@ -56,6 +60,7 @@ gint debug_skin = FALSE;
 /* list of all windows created with ui_new() */
 static GList *slik_ui_list = NULL;
 
+
 /*
  *-------------
  * misc
@@ -106,6 +111,31 @@ void ui_iconify(UIData *ui)
         XIconifyWindow (GDK_DISPLAY (), xwindow, DefaultScreen (GDK_DISPLAY ()));
 }
 
+static gint ui_mode_unique(UIData *ui, const gchar *mode_key)
+{
+	GList *work;
+
+	if (!mode_key) return FALSE;
+
+	if (ui->parent) ui = ui->parent;
+
+	if (strcmp(mode_key, "skindata") == 0) return FALSE;
+	if (ui->skin_mode_key && strcmp(ui->skin_mode_key, mode_key) == 0) return FALSE;
+
+	work = ui->children;
+	while (work)
+		{
+		UIData *child;
+
+		child = work->data;
+		work = work->next;
+
+		if (child->skin_mode_key && strcmp(child->skin_mode_key, mode_key) == 0) return FALSE;
+		}
+
+	return TRUE;
+}
+
 /*
  *-------------
  * these are the reserved widgets for window resizing and skin mode toggles.
@@ -126,6 +156,16 @@ static void ui_skin_toggle_click_cb(ButtonData *button, const gchar *key, gpoint
 	ui_skin_mode_set(ui, mode_key);
 }
 
+static gint ui_skin_expand_status_cb(ButtonData *button, const gchar *key, gpointer data)
+{
+	UIData *ui = data;
+	SkinData *skin;
+
+	skin = ui->skin;
+
+	return (skin->width != skin->width_def || skin->height != skin->height_def);
+}
+
 static void ui_skin_expand_click_cb(ButtonData *button, const gchar *key, gpointer data)
 {
 	UIData *ui = data;
@@ -168,6 +208,9 @@ static void ui_skin_open_click_cb(ButtonData *button, const gchar *key, gpointer
 {
 	UIData *ui = data;
 	const gchar *mode_key;
+	gchar *datafile;
+	UIData *child;
+	SkinData *skin;
 
 	mode_key = ui_widget_get_data_by_widget(ui, button);
 
@@ -177,21 +220,59 @@ static void ui_skin_open_click_cb(ButtonData *button, const gchar *key, gpointer
 		return;
 		}
 
+	if (!ui_mode_unique(ui, mode_key)) return;
+
 	if (debug_mode) printf("ui2_main.c: opening skin \"%s\"for addition\n", mode_key);
+
+	datafile = g_strconcat(ui->skin_path, "/", (mode_key) ? mode_key : "skindata", NULL);
+	skin = skin_parse(ui->skin_path, datafile, FALSE);
+	g_free(datafile);
+
+	if (!skin)
+		{
+		printf("ui2_main.c: failed to open skin \"%s\"for new window\n", mode_key);
+		return;
+		}
+
+	if (ui->parent) ui = ui->parent;
+
+	child = ui_new(ui->class, mode_key, ui->decorations, ui->class);
+	child->allow_move = ui->allow_move;
+
+	/* copy a few things from parent, it is up to the application to determine if
+	 * the UI in the callback is the parent.
+	 */
+	child->click_func = ui->click_func;
+	child->click_data = ui->click_data;
+
+	ui_group_set_child(ui, child);
+
+	ui_skin_set(child, skin, ui->skin_path, mode_key);
+
+	if (ui->new_window_func)
+		{
+		ui->new_window_func(child, child->key, ui->new_window_data);
+		}
+
+	gtk_widget_show(child->window);
 }
 
 static void ui_skin_close_click_cb(ButtonData *button, const gchar *key, gpointer data)
 {
 	UIData *ui = data;
 
+	if (!ui->parent) return;
+
 	if (debug_mode) printf("ui2_main.c: closing skin \"%s\"\n", ui->skin_mode_key);
+
+	ui_close(ui);
 }
 
 static void ui_skin_iconify_click_cb(ButtonData *button, const gchar *key, gpointer data)
 {
 	UIData *ui = data;
 
-	ui_iconify(ui);
+	if (ui->allow_move) ui_iconify(ui);
 }
 
 static void ui_add_reserved(UIData *ui)
@@ -216,7 +297,7 @@ static void ui_add_reserved(UIData *ui)
 	if (!ui_registered_key_exists(ui, "skin_expand", button_type_id()))
 		{
 		button_register_key("skin_expand", ui,
-				     NULL, NULL,
+				     ui_skin_expand_status_cb, ui,
 				     ui_skin_expand_click_cb, ui,
 				     NULL, NULL,
 				     NULL, NULL);
@@ -265,6 +346,7 @@ void ui_free(UIData *ui)
 	skin_free(ui->skin);
 	g_free(ui->skin_path);
 	g_free(ui->skin_mode_key);
+	g_free(ui->class);
 	g_free(ui->key);
 	g_free(ui);
 }
@@ -274,6 +356,11 @@ static void ui_show_cb(GtkWidget *widget, gpointer data)
 	UIData *ui = data;
 	gint x, y;
 
+	if (!ui->skin)
+		{
+		printf("warning: %s shown with a NULL skin\n", ui->key);
+		}
+
 	if (slik_remember_position &&
 	    ui && ui->allow_move && ui->skin &&
 	    ui_state_get(filename_from_path(ui->skin_path), ui->skin_mode_key, &x, &y, NULL, NULL))
@@ -290,7 +377,8 @@ static void ui_show_cb(GtkWidget *widget, gpointer data)
 		}
 
 	/* only do this once */
-	gtk_signal_disconnect_by_func(GTK_OBJECT(widget), ui_show_cb, data);
+	gtk_signal_disconnect_by_func(GTK_OBJECT(widget),
+				      GTK_SIGNAL_FUNC(ui_show_cb), data);
 }
 
 static void ui_hide_cb(GtkWidget *widget, gpointer data)
@@ -314,32 +402,36 @@ static void ui_destroy_cb(GtkWidget *widget, gpointer data)
 	ui_free(ui);
 }
 
-static UIData *ui_real_new(const gchar *class, const gchar *subclass,
+static UIData *ui_real_new(const gchar *class, const gchar *key,
 			   gint decorations, const gchar *title, GtkWindowType type)
 {
 	UIData *ui;
 
-	/* make sure widgets are registered */
-	ui_init_types();
-
 	ui = g_new0(UIData, 1);
 
 	ui->decorations = decorations;
 	ui->root_win_idle = -1;
 	ui->edit = NULL;
 	ui->skin = NULL;
-	ui->key = g_strdup(subclass);
+	ui->class = g_strdup((class) ? class : "SLIK");
+	ui->key = g_strdup(key);
 	ui->allow_move = TRUE;
 
+	ui->parent = NULL;
+	ui->children = NULL;
+
 	ui->window = gtk_window_new(type);
 	gtk_window_set_policy(GTK_WINDOW(ui->window), FALSE, FALSE, TRUE);
 	gtk_container_border_width (GTK_CONTAINER(ui->window), 0);
-	gtk_window_set_wmclass(GTK_WINDOW(ui->window), subclass, class);
+	gtk_window_set_wmclass(GTK_WINDOW(ui->window), ui->key, ui->class);
 	gtk_window_set_title(GTK_WINDOW(ui->window), title);
 
-	gtk_signal_connect(GTK_OBJECT(ui->window), "show", ui_show_cb, ui);
-	gtk_signal_connect(GTK_OBJECT(ui->window), "hide", ui_hide_cb, ui);
-	gtk_signal_connect(GTK_OBJECT(ui->window), "destroy", ui_destroy_cb, ui);
+	gtk_signal_connect(GTK_OBJECT(ui->window), "show",
+			   GTK_SIGNAL_FUNC(ui_show_cb), ui);
+	gtk_signal_connect(GTK_OBJECT(ui->window), "hide",
+			   GTK_SIGNAL_FUNC(ui_hide_cb), ui);
+	gtk_signal_connect(GTK_OBJECT(ui->window), "destroy",
+			   GTK_SIGNAL_FUNC(ui_destroy_cb), ui);
 
 	gtk_widget_realize(ui->window);
 
@@ -348,6 +440,8 @@ static UIData *ui_real_new(const gchar *class, const gchar *subclass,
 	gtk_container_add(GTK_CONTAINER(ui->window), ui->display);
 	gtk_widget_show(ui->display);
 
+	if (GTK_WIDGET_CAN_FOCUS(ui->display)) gtk_widget_grab_focus(ui->display);
+
 	gtk_widget_realize(ui->display);
 
 	if (type != GTK_WINDOW_POPUP && !ui->decorations) gdk_window_set_decorations(ui->window->window, 0);
@@ -370,17 +464,66 @@ UIData *ui_new_dialog(const gchar *class, const gchar *subclass, gint decoration
 	return ui_real_new(class, subclass, decorations, title, GTK_WINDOW_DIALOG);
 }
 
+UIData *ui_new_into_container(const gchar *class, const gchar *key, GtkWidget *widget)
+{
+	UIData *ui;
+
+	ui = g_new0(UIData, 1);
+
+	ui->decorations = FALSE;
+	ui->root_win_idle = -1;
+	ui->edit = NULL;
+	ui->skin = NULL;
+	ui->class = g_strdup((class) ? class : "SLIK");
+	ui->key = g_strdup(key);
+	ui->allow_move = FALSE;
+
+	ui->parent = NULL;
+	ui->children = NULL;
+
+	ui->window = NULL;
+
+	ui->display = gtk_drawing_area_new();
+	gtk_signal_connect(GTK_OBJECT(ui->display), "destroy",
+			   GTK_SIGNAL_FUNC(ui_destroy_cb), ui);
+
+	ui_display_events_init(ui);
+	if (widget)
+		{
+		gtk_container_add(GTK_CONTAINER(widget), ui->display);
+		gtk_widget_realize(ui->display);
+		}
+
+	gtk_widget_show(ui->display);
+
+	/* add the default reserved keys */
+	ui_add_reserved(ui);
+
+	slik_ui_list = g_list_append(slik_ui_list, ui);
+
+	return ui;
+}
+
 static gint ui_close_cb(gpointer data)
 {
 	UIData *ui = data;
 
-	gtk_widget_destroy(ui->window);
+	if (ui->window)
+		{
+		gtk_widget_destroy(ui->window);
+		}
+	else
+		{
+		gtk_widget_destroy(ui->display);
+		}
 
 	return FALSE;
 }
 
 void ui_close(UIData *ui)
 {
+	GList *work;
+
 	if (!ui || ui->destroyed) return;
 
 	/* This may be called in the middle of a widget event (like a button press handler),
@@ -389,7 +532,28 @@ void ui_close(UIData *ui)
 	 */
 
 	ui->destroyed = TRUE;
-	gtk_widget_hide(ui->window);
+	if (ui->window && GTK_WIDGET_VISIBLE(ui->window)) gtk_widget_hide(ui->window);
+
+	/* close children */
+	work = ui->children;
+	while (work)
+		{
+		UIData *child = work->data;
+		work = work->next;
+
+		ui_close(child);
+		}
+
+	/* if a child, break from parent */
+	if (ui->parent)
+		{
+		UIData *parent;
+
+		parent = ui->parent;
+
+		parent->children = g_list_remove(parent->children, ui);
+		ui->parent = NULL;
+		}
 
 	gtk_idle_add(ui_close_cb, ui);
 }
@@ -400,6 +564,25 @@ void ui_set_mouse_callback(UIData *ui, void (*func)(UIData *ui, gint button, gui
 	ui->click_data = data;
 }
 
+void ui_set_skin_callback(UIData *ui, SkinData *(*func)(UIData *ui, const gchar *key, gpointer data), gpointer data)
+{
+	ui->skin_func = func;
+	ui->skin_data = data;
+}
+
+void ui_set_back_callback(UIData *ui, gint (*func)(UIData *ui, GdkPixbuf *pixbuf, gpointer data), gpointer data)
+{
+	ui->back_func = func;	
+	ui->back_data = data;
+}
+
+void ui_set_new_window_callback(UIData *ui, void (*func)(UIData *ui, const gchar *key, gpointer data), gpointer data)
+{
+	ui->new_window_func = func;
+	ui->new_window_data = data;
+}
+
+
 /*
  *-------------
  * app side keys
@@ -496,7 +679,7 @@ static RegisterData *ui_registered_by_key(UIData *ui, const gchar *key, WidgetTy
 	GList *work;
 
 	work = ui->register_list;
-	while(work)
+	while (work)
 		{
 		RegisterData *rd = work->data;
 		if (rd->type == type && strcmp(rd->key, key) == 0) return rd;
@@ -507,15 +690,16 @@ static RegisterData *ui_registered_by_key(UIData *ui, const gchar *key, WidgetTy
 
 gpointer ui_get_registered_callbacks(UIData *ui, const gchar *key, WidgetType type)
 {
-	GList *work;
+	RegisterData *rd;
 
-	work = ui->register_list;
-	while(work)
+	rd = ui_registered_by_key(ui, key, type);
+	if (rd) return rd->callbacks;
+
+	if (ui->parent)
 		{
-		RegisterData *rd = work->data;
-		if (rd->type == type && strcmp(rd->key, key) == 0) return rd->callbacks;
-		work = work->next;
+		return ui_get_registered_callbacks(ui->parent, key, type);
 		}
+
 	return NULL;
 }
 
@@ -531,18 +715,32 @@ gint ui_widget_exists(UIData *ui, const gchar *key, WidgetType type)
 	return (skin_widget_get_by_key(ui->skin, key, type) != NULL);
 }
 
-void ui_debug_print_registered_keys(UIData *ui)
+gint ui_registered_key_is_private(UIData *ui, const gchar *key, WidgetType type)
 {
 	GList *work;
 
-	printf("UI is \"%s\"\n", ui->skin_path);
-	printf("Key is \"%s\"\n", ui->skin_mode_key);
-
-	printf("UI registered keys (%3d):\n", g_list_length(ui->register_list));
-	printf("=========================\n");
+	/* walk the loop ourselves, for speed only strcmp if private and type match */
 
 	work = ui->register_list;
-	while(work)
+	while (work)
+		{
+		RegisterData *rd = work->data;
+		if (rd->private && rd->type == type && strcmp(rd->key, key) == 0) return TRUE;
+		work = work->next;
+		}
+	return FALSE;
+}
+
+static void ui_debug_print_register_list(UIData *ui)
+{
+	GList *work;
+
+	printf("-------------------------\n");
+	printf("UI registered keys (%3d):\n", g_list_length(ui->register_list));
+	printf("-[key]------------------------[type]-----\n");
+
+	work = ui->register_list;
+	while (work)
 		{
 		RegisterData *rd = work->data;
 		work = work->next;
@@ -551,10 +749,80 @@ void ui_debug_print_registered_keys(UIData *ui)
 		}
 }
 
+void ui_debug_print_registered_keys(UIData *ui)
+{
+	if (!ui) return;
+
+	if (ui->parent)
+		{
+		printf("=========================\n");
+		printf("UI is \"%s\" is a child of \"%s\"\n", ui->key, ui->parent->key);
+		printf("printing out keys starting at the parent\n");
+		printf("-------------------------\n");
+		ui_debug_print_registered_keys(ui->parent);
+		return;
+		}
+
+	printf("=========================\n");
+	printf("UI is \"%s\"\n", ui->key);
+	printf("-------------------------\n");
+	printf("    skin: \"%s\"\n", ui->skin_path);
+	printf("mode key: \"%s\"\n", ui->skin_mode_key);
+
+	ui_debug_print_register_list(ui);
+
+	if (ui->children)
+		{
+		GList *work;
+
+		printf("-------------------------\n");
+		printf("children: %d\n", g_list_length(ui->children));
+
+		work = ui->children;
+		while (work)
+			{
+			UIData *child;
+
+			child = work->data;
+			work = work->next;
+
+			printf("=========================\n");
+			printf("   child: \"%s\"\n", child->key);
+			printf("mode key: \"%s\"\n", child->skin_mode_key);
+
+			ui_debug_print_register_list(child);
+			}
+		}
+
+	printf("=========================\n");
+}
+
 void ui_debug_print_all_keys(UIData *ui)
 {
+	GList *work;
+
 	ui_debug_print_registered_keys(ui);
+
+	if (ui->parent) ui = ui->parent;
+
 	skin_debug_print_registered_keys(ui->skin);
+
+	work = ui->children;
+	while (work)
+		{
+		UIData *child;
+
+		child = work->data;
+		work = work->next;
+
+		printf("=========================\n");
+		printf("child: \"%s\"\n", child->key);
+		printf("-------------------------\n");
+
+		skin_debug_print_registered_keys(child->skin);
+		}
+
+	printf("=========================\n");
 }
 
 /*
@@ -569,7 +837,7 @@ gint ui_geometry_get(UIData *ui, gint *x, gint *y, gint *w, gint *h)
 
 	if (!ui || !ui->skin ||
 	    !ui->window || !ui->window->window ||
-	    !GTK_WIDGET_VISIBLE(ui->window)) return FALSE;
+	    !GTK_WIDGET_REALIZED(ui->window)) return FALSE;
 
 	gdk_window_get_position(ui->window->window, &rx, &ry);
 	gdk_window_get_size(ui->window->window, &rw, &rh);
@@ -600,6 +868,31 @@ void ui_skin_set(UIData *ui, SkinData *skin, const gchar *path, const gchar *mod
 
 	if (!skin) return;
 
+	/* handle children, closing any that were opened
+	 * with the skin_open button, these are identified by
+	 * having the same key and skin_mode_key.
+	 */
+	if (( (path == NULL) != (ui->skin_path == NULL) ) ||
+	    (path && strcmp(path, ui->skin_path) != 0) )
+		{
+		GList *work;
+
+		work = ui->children;
+		while (work)
+			{
+			UIData *child;
+
+			child = work->data;
+			work = work->next;
+
+			if (child->key && child->skin_mode_key &&
+			    strcmp(child->key, child->skin_mode_key) == 0)
+				{
+				ui_close(child);
+				}
+			}
+		}
+
 	/* save current state of old skin */
 	ui_geometry_get(ui, NULL, NULL, NULL, NULL);
 
@@ -610,6 +903,9 @@ void ui_skin_set(UIData *ui, SkinData *skin, const gchar *path, const gchar *mod
 	ui->skin = skin;
 	skin->ui = ui;
 
+	ui->active_widget = NULL;
+	ui->focus_widget = NULL;
+
 	/* use tmp, may be src = dest */
 
 	tmp = g_strdup(path);
@@ -647,6 +943,32 @@ void ui_skin_set(UIData *ui, SkinData *skin, const gchar *path, const gchar *mod
 	ui_display_sync_all(ui);
 }
 
+SkinData *ui_skin_load_default(UIData *ui, const gchar *key)
+{
+	SkinData *skin = NULL;
+
+	if (ui->skin_func)
+		{
+		skin = ui->skin_func(ui, key, ui->skin_data);
+		}
+	if (!skin)
+		{
+		/* well, return something! */
+		skin = skin_new();
+
+		skin->real_overlay = gdk_pixbuf_new_from_xpm_data(ui_slik_logo());
+		skin->width = gdk_pixbuf_get_width(skin->real_overlay);
+		skin->height = gdk_pixbuf_get_height(skin->real_overlay);
+
+		/* add some widgets (a text message maybe? ) */
+
+		skin->width_def = skin->width_min = skin->width_max = skin->width;
+		skin->height_def = skin->height_min = skin->height_max = skin->height;
+		}
+
+	return skin;
+}
+
 gint ui_skin_load(UIData *ui, const gchar *path, const gchar *mode_key)
 {
 	SkinData *skin;
@@ -686,7 +1008,7 @@ gint ui_skin_load(UIData *ui, const gchar *path, const gchar *mode_key)
 		}
 	else
 		{
-		skin = skin_load_default(ckey);
+		skin = ui_skin_load_default(ui, ckey);
 		}
 
 	if (skin)
@@ -756,6 +1078,14 @@ gint ui_skin_mode_set(UIData *ui, const gchar *mode_key)
 	return success;
 }
 
+void ui_set_underlay(UIData *ui, GdkPixbuf *pb)
+{
+	if (!ui || !ui->skin) return;
+
+	skin_set_underlay(ui->skin, ui, pb);
+	ui_display_sync(ui, FALSE);
+}
+
 void ui_sync_states(void)
 {
 	GList *work;
@@ -799,6 +1129,29 @@ UIData *ui_find_by_key(const gchar *key)
 	return NULL;
 }
 
+/*
+ *-------------
+ * grouping (children) support
+ *-------------
+ */
+
+void ui_group_set_child(UIData *parent, UIData *child)
+{
+	if (!parent || !child) return;
+
+	if (child->parent) return;
+
+	child->parent = parent;
+	parent->children = g_list_append(parent->children, child);
+}
+
+UIData *ui_group_get_parent(UIData *ui)
+{
+	if (!ui) return NULL;
+
+	return ui->parent;
+}
+
 /*
  *-------------
  * SLIK credits / about / logo
@@ -856,7 +1209,8 @@ static void ui_slik_about(void)
 		}
 
 	slik_about = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	gtk_signal_connect(GTK_OBJECT(slik_about), "delete_event", GTK_SIGNAL_FUNC(ui_slik_about_delete_cb), NULL);
+	gtk_signal_connect(GTK_OBJECT(slik_about), "delete_event",
+			   GTK_SIGNAL_FUNC(ui_slik_about_delete_cb), NULL);
 	gtk_window_set_wmclass(GTK_WINDOW(slik_about), "about", "SLIK");
 	gtk_window_set_title(GTK_WINDOW(slik_about), _("About - SLIK"));
 	gtk_container_border_width(GTK_CONTAINER(slik_about), 5);
@@ -895,14 +1249,16 @@ static void ui_slik_about(void)
 	button = gtk_toggle_button_new();
 	gtk_widget_set_usize(button, 8, 8);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), debug_mode);
-	gtk_signal_connect(GTK_OBJECT(button), "toggled", slik_about_toggle_cb, &debug_mode);
+	gtk_signal_connect(GTK_OBJECT(button), "toggled",
+			   GTK_SIGNAL_FUNC(slik_about_toggle_cb), &debug_mode);
 	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 4);
 	gtk_widget_show(button);
 
 	button = gtk_toggle_button_new();
 	gtk_widget_set_usize(button, 8, 8);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), debug_skin);
-	gtk_signal_connect(GTK_OBJECT(button), "toggled", slik_about_toggle_cb, &debug_skin);
+	gtk_signal_connect(GTK_OBJECT(button), "toggled",
+			   GTK_SIGNAL_FUNC(slik_about_toggle_cb), &debug_skin);
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 4);
 	gtk_widget_show(button);
 
@@ -926,7 +1282,8 @@ GtkWidget *ui_slik_credit(void)
 	frame = gtk_button_new();
 	gtk_button_set_relief(GTK_BUTTON(frame), GTK_RELIEF_NONE);
 	gtk_container_border_width(GTK_CONTAINER(frame), 10);
-	gtk_signal_connect(GTK_OBJECT(frame), "clicked", GTK_SIGNAL_FUNC(ui_slik_about), NULL);
+	gtk_signal_connect(GTK_OBJECT(frame), "clicked",
+			   GTK_SIGNAL_FUNC(ui_slik_about), NULL);
 
 	hbox = gtk_hbox_new(FALSE, 5);
 	gtk_container_add(GTK_CONTAINER(frame), hbox);
@@ -955,7 +1312,3 @@ GtkWidget *ui_slik_credit(void)
 }
 
 
-
-
-
-
