@@ -48,11 +48,18 @@ WidgetType digit_type_id(void);
 void digit_type_init(void);
 
 WidgetData *digit_register(SkinData *skin, DigitData *digit, const gchar *key, const gchar *text_id);
+WidgetData *digit_register_to_skin(const gchar *key, gchar **data,
+				   SkinData *skin, const gchar *text_id);
 
-NumberData *number_new(DigitData *digit, gint length, gint zeros, gint centered, gint x, gint y);
+NumberData *number_new(DigitData *digit, gint x, gint y,
+		       gint length, gint zeros, gint centered);
 void number_free(NumberData *number);
 
 WidgetData *number_register(SkinData *skin, NumberData *number, const gchar *key, const gchar *text_id);
+WidgetData *number_register_to_skin(const gchar *key, DigitData *digit, gint x, gint y,
+				    gint length, gint zeros, gint centered,
+				    SkinData *skin, const gchar *text_id);
+
 RegisterData *number_register_key(const gchar *key, UIData *ui,
 				  gint (*status_get_func)(NumberData *number, const gchar *key, gpointer data), gpointer status_get_data);
 gint number_value_get(const gchar *key, UIData *ui);
