@@ -311,13 +311,16 @@ static gpointer number_edit_props(UIData *ui, WidgetData *wd, GtkWidget *vbox, g
 		hbox = ui_edit_frame_new(vbox, FALSE, NULL);
 
 		nd->spin_length = ui_edit_spin_new(hbox, _("Length:"), 1, 8,  &adj);
-		gtk_signal_connect(adj, "value_changed", number_edit_props_length_cb, nd);
+		gtk_signal_connect(adj, "value_changed",
+				   GTK_SIGNAL_FUNC(number_edit_props_length_cb), nd);
 
 		nd->zero_button = ui_edit_toggle_new(vbox, _("Pad with zeros"));
-		gtk_signal_connect(GTK_OBJECT(nd->zero_button), "clicked", number_edit_props_alter_cb, nd);
+		gtk_signal_connect(GTK_OBJECT(nd->zero_button), "clicked",
+				   GTK_SIGNAL_FUNC(number_edit_props_alter_cb), nd);
 
 		nd->center_button = ui_edit_toggle_new(vbox, _("Center"));
-		gtk_signal_connect(GTK_OBJECT(nd->center_button), "clicked", number_edit_props_alter_cb, nd);
+		gtk_signal_connect(GTK_OBJECT(nd->center_button), "clicked",
+				   GTK_SIGNAL_FUNC(number_edit_props_alter_cb), nd);
 
 		hbox = ui_edit_frame_new(vbox, FALSE, NULL);
 
@@ -412,7 +415,7 @@ static void number_edit_page_add_cb(GtkWidget *widget, gpointer data)
 		}
 
 	if (!digit) return;
-	number = number_new(digit, length, zeros, center, 0, 0);
+	number = number_new(digit, 0, 0, length, zeros, center);
 	
 	if (number)
 		{
@@ -495,17 +498,20 @@ static GtkWidget *number_edit_page_new(EditData *ed)
 	pd->length_spin = ui_edit_spin_new(vbox, _("Length:"), 1, 8, NULL);
 	pd->zeros_button = ui_edit_toggle_new(vbox, _("Pad with zeros"));
 	pd->center_button = ui_edit_toggle_new(vbox, _("Center"));
-	gtk_signal_connect(GTK_OBJECT(pd->center_button), "clicked", number_edit_page_center_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(pd->center_button), "clicked",
+			   GTK_SIGNAL_FUNC(number_edit_page_center_cb), pd);
 
 	pd->digit_id_entry = ui_edit_entry_new(vbox, _("Digit key:"));
-	gtk_signal_connect(GTK_OBJECT(pd->digit_id_entry), "changed", number_edit_page_digit_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(pd->digit_id_entry), "changed",
+			   GTK_SIGNAL_FUNC(number_edit_page_digit_cb), pd);
 	pd->image_entry = ui_edit_path_entry_new(vbox, _("Image:"), "SLIK_number_image");
 	pd->pixmap = ui_edit_pixmap_new(vbox);
 	ui_edit_path_entry_connect_pixmap(pd->image_entry, pd->pixmap);
 
 	button = gtk_button_new_with_label(_("Add"));
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", number_edit_page_add_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(number_edit_page_add_cb), pd);
 	gtk_widget_show(button);
 
 	sep = gtk_hseparator_new();
@@ -517,11 +523,13 @@ static GtkWidget *number_edit_page_new(EditData *ed)
 
 	button = gtk_button_new_with_label(_("Add"));
 	gtk_box_pack_end(GTK_BOX(frame), button, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", number_edit_page_digit_add_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(number_edit_page_digit_add_cb), pd);
 	gtk_widget_show(button);
 
 	pd->clist = ui_edit_clist_new(hbox, titles, 2);
-	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row", number_edit_page_clist_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row",
+			   GTK_SIGNAL_FUNC(number_edit_page_clist_cb), pd);
 
 	/* hacky, but it works */
 	ui_edit_widget_id_list_set_page(ed->widget_id_list, digit_type_id(), hbox);
