@@ -23,6 +23,11 @@
 #include "ui2_widget.h"
 #include "ui_pixbuf_ops.h"
 
+#include <gdk/gdkkeysyms.h> /* for key values */
+
+
+#define SLIDER_KEY_RELEASE_TIME 333
+
 
 typedef struct _SliderCallbackData SliderCallbackData;
 struct _SliderCallbackData
@@ -38,22 +43,36 @@ struct _SliderCallbackData
 	gpointer slider_drag_data;
 };
 
+typedef struct _SliderSetData SliderSetData;
+struct _SliderSetData
+{
+	gfloat step_size;
+	gint page_size;
+};
+
 static WidgetType type_id = -1;
 
 
+static void slider_key_movement_reset_timer(SliderData *slider, gint start);
+
+
 /*
  *-----------------------------
  * new / free
  *-----------------------------
  */
 
-SliderData *slider_new(GdkPixbuf *pb, gint has_press, gint prelight, gint vertical, gint reversed,
-		       gint length, gint x, gint y, const gchar *item_key)
+SliderData *slider_new(GdkPixbuf *pb, gint x, gint y,
+		       gint length, gint vertical, gint reversed,
+		       gint has_press, gint prelight,
+		       const gchar *item_key)
 {
 	SliderData *slider;
 	gint width;
 	gint height;
 
+	slider_type_init();
+
 	if (!pb) return NULL;
 
 	util_size(&x);
@@ -109,27 +128,35 @@ SliderData *slider_new(GdkPixbuf *pb, gint has_press, gint prelight, gint vertic
 	slider->border_2 = 0;
 	slider->stretch = FALSE;
 
+	slider->step_size = 0.01;
+	slider->page_size = 10;
+	slider->key_timer_id = -1;
+
 	return slider;
 }
 
-SliderData *slider_new_from_data(gchar **data, gint has_press, gint prelight, gint vertical, gint reversed,
-				 gint length, gint x, gint y, const gchar *item_key)
+SliderData *slider_new_from_data(gchar **data, gint x, gint y,
+				 gint length, gint vertical, gint reversed,
+				 gint has_press, gint prelight,
+				 const gchar *item_key)
 {
 	GdkPixbuf *pb;
 
 	pb = gdk_pixbuf_new_from_xpm_data((const char **)data);
 
-	return slider_new(pb, has_press, prelight, vertical, reversed, length, x, y, item_key);
+	return slider_new(pb, x, y, length, vertical, reversed, has_press, prelight, item_key);
 }
 
-SliderData *slider_new_from_file(const gchar *file, gint has_press, gint prelight, gint vertical, gint reversed,
-			gint length, gint x, gint y, const gchar *item_key)
+SliderData *slider_new_from_file(const gchar *file, gint x, gint y,
+				 gint length, gint vertical, gint reversed,
+				 gint has_press, gint prelight,
+				 const gchar *item_key)
 {
 	GdkPixbuf *pb;
 
 	pb = gdk_pixbuf_new_from_file(file);
 
-	return slider_new(pb, has_press, prelight, vertical, reversed, length, x, y, item_key);
+	return slider_new(pb, x, y, length, vertical, reversed, has_press, prelight, item_key);
 }
 
 void slider_set_attributes(SliderData *slider, gint sizeable,
@@ -164,6 +191,8 @@ void slider_free(SliderData *slider)
 {
 	if (!slider) return;
 
+	slider_key_movement_reset_timer(slider, FALSE);
+
 	if (slider->pixbuf) gdk_pixbuf_unref(slider->pixbuf);
 	if (slider->overlay) gdk_pixbuf_unref(slider->overlay);
 	g_free(slider->item_key);
@@ -273,8 +302,8 @@ static void slider_draw_real(SliderData *slider, gfloat value, gint move,
 
 		if (slider->item_key)
 			{
-			ui_widget_for_each_key(ui, slider->item_key, item_type_id(),
-					       slider_force_item_draw_cb, slider);
+			ui_widget_for_each_key_one(ui, slider->item_key, item_type_id(),
+						   slider_force_item_draw_cb, slider);
 			}
 
 		if (slider->vertical)
@@ -312,7 +341,7 @@ static void slider_draw_real(SliderData *slider, gfloat value, gint move,
 				}
 			}
 
-		ui_display_render_area(ui, rx, ry, rw, rh);
+		ui_display_render_area(ui, rx, ry, rw, rh, slider->wd);
 		}
 }
 
@@ -396,7 +425,7 @@ static void slider_motion(gpointer data, const gchar *key, gint x, gint y, GdkPi
 {
 	SliderData *slider = data;
 
-	if (!slider) return;
+	if (!slider || slider->key_timer_id != -1) return;
 
 	if (slider->pushed)
 		{
@@ -434,7 +463,7 @@ static gint slider_press(gpointer data, const gchar *key, gint x, gint y, GdkPix
 {
 	SliderData *slider = data;
 
-	if (!slider) return FALSE;
+	if (!slider || slider->key_timer_id != -1) return FALSE;
 
 	if (slider_test_proximity(slider, x, y))
 		{
@@ -462,7 +491,7 @@ static void slider_release(gpointer data, const gchar *key, gint x, gint y, GdkP
 	SliderCallbackData *cd;
 	gfloat val;
 
-	if (!slider || !slider->pushed) return;
+	if (!slider || !slider->pushed || slider->key_timer_id != -1) return;
 
 	val = slider_calc_value_by_coord(slider, x, y);
 
@@ -533,6 +562,174 @@ static void slider_back_set(gpointer data, GdkPixbuf *pb)
 		}
 }
 
+static gint slider_key_release_cb(gpointer data)
+{
+	SliderData *slider = data;
+	UIData *ui;
+	GdkPixbuf *pb;
+
+	ui = skin_get_ui(slider->skin);
+	pb = skin_get_pixbuf(slider->skin);
+
+	if (slider->pushed)
+		{
+		SliderCallbackData *cd;
+		slider_draw_real(slider, 0.0, FALSE, FALSE, FALSE, FALSE, pb, ui);
+
+		cd = ui_get_registered_callbacks(ui, slider->wd->key, type_id);
+
+		if (cd && cd->slider_release_func)
+			{
+			cd->slider_release_func(slider, slider->wd->key, slider_value_public(slider), cd->slider_release_data);
+			}
+		}
+
+	slider->key_timer_id = -1;
+	return FALSE;
+}
+
+static void slider_key_movement_reset_timer(SliderData *slider, gint start)
+{
+	if (slider->key_timer_id != -1)
+		{
+		gtk_timeout_remove(slider->key_timer_id);
+		slider->key_timer_id = -1;
+		}
+
+	if (start)
+		{
+		slider->key_timer_id = gtk_timeout_add(SLIDER_KEY_RELEASE_TIME, slider_key_release_cb, slider);
+		}
+}
+
+static void slider_key_movement(SliderData *slider, const gchar *key, gfloat value, GdkPixbuf *pb, UIData *ui)
+{
+	SliderCallbackData *cd;
+	gint was_pushed;
+
+	value = CLAMP(slider->value + value, 0.0, 1.0);
+	if (value == slider->value) return;
+
+	was_pushed = slider->pushed;
+	slider_draw_real(slider, value, TRUE, FALSE, TRUE, FALSE, pb, ui);
+
+	cd = ui_get_registered_callbacks(ui, key, type_id);
+
+	if (!was_pushed)
+		{
+		if (cd && cd->slider_press_func)
+			{
+			cd->slider_press_func(slider, key, slider_value_public(slider), cd->slider_press_data);
+			}
+		}
+	else
+		{
+		if (cd && cd->slider_drag_func)
+			{
+			cd->slider_drag_func(slider, key, slider_value_public(slider), cd->slider_drag_data);
+			}
+		}
+
+	slider_key_movement_reset_timer(slider, TRUE);
+}
+
+static gint slider_key_event_cb(gpointer widget, const gchar *key, GdkEventKey *event, GdkPixbuf *pb, UIData *ui)
+{
+	SliderData *slider = widget;
+	gint ret = FALSE;
+
+	switch (event->keyval)
+		{
+		case GDK_Up:
+		case GDK_KP_Up:
+			if (slider->vertical)
+				{
+				slider_key_movement(slider, key, 0 - slider->step_size, pb, ui);
+				ret = TRUE;
+				}
+			break;
+		case GDK_Down:
+		case GDK_KP_Down:
+			if (slider->vertical)
+				{
+				slider_key_movement(slider, key, slider->step_size, pb, ui);
+				ret = TRUE;
+				}
+			break;
+		case GDK_Left:
+		case GDK_KP_Left:
+			if (!slider->vertical)
+				{
+				slider_key_movement(slider, key, 0 - slider->step_size, pb, ui);
+				ret = TRUE;
+				}
+			break;
+		case GDK_Right:
+		case GDK_KP_Right:
+			if (!slider->vertical)
+				{
+				slider_key_movement(slider, key, slider->step_size, pb, ui);
+				ret = TRUE;
+				}
+			break;
+		case GDK_Page_Up:
+		case GDK_KP_Page_Up:
+			slider_key_movement(slider, key,
+					    (0 - slider->step_size) * (gfloat)slider->page_size, pb, ui);
+			ret = TRUE;
+			break;
+		case GDK_Page_Down:
+		case GDK_KP_Page_Down:
+			slider_key_movement(slider, key,
+					    slider->step_size * (gfloat)slider->page_size, pb, ui);
+			ret = TRUE;
+			break;
+		case GDK_Home:
+		case GDK_KP_Home:
+			slider_key_movement(slider, key, -1.0, pb, ui);
+			ret = TRUE;
+			break;
+		case GDK_End:
+		case GDK_KP_End:
+			slider_key_movement(slider, key, 1.0, pb, ui);
+			ret = TRUE;
+			break;
+		default:
+			break;
+		}
+
+	return ret;
+}
+
+static void slider_handle_geometry(SliderData *slider, gint *x, gint *y, gint *w, gint *h)
+{
+	if (slider->vertical)
+		{
+		*x = slider->x;
+		*y = slider->y + slider->position;
+		}
+	else
+		{
+		*x = slider->x + slider->position;
+		*y = slider->y;
+		}
+	*w = slider->handle_width;
+	*h = slider->handle_height;
+}
+
+static gint slider_focus_draw_cb(gpointer widget, const gchar *key,
+				 gint x, gint y, gint w, gint h, GdkPixbuf *pb, UIData *ui)
+{
+	SliderData *slider = widget;
+	gint hx, hy, hw, hh;
+
+	slider_handle_geometry(slider, &hx, &hy, &hw, &hh);
+	ui_display_draw_focus(ui, pb, hx, hy, hw, hh,
+			      x, y, w, h, NULL);
+
+	return TRUE;
+}
+
 static gint slider_get_geometry(gpointer widget, gint *x, gint *y, gint *w, gint *h)
 {
 	SliderData *slider = widget;
@@ -612,8 +809,9 @@ static WidgetData *slider_parse(SkinData *skin, GList *list, const gchar *skin_d
 	if (!key_list_read_int(list, "border_2", &border_2)) border_2 = 0;
 	stretch = key_list_read_bool(list, "stretch");
 
-	slider = slider_new_from_file(filename, has_press, prelight, vertical, reverse,
-				      length, x, y, item_key);
+	slider = slider_new_from_file(filename, x, y,
+				      length, vertical, reverse,
+				      has_press, prelight, item_key);
 	slider_set_attributes(slider, sizeable, trough_length, border_1, border_2, stretch);
 
 	if (slider)
@@ -646,7 +844,26 @@ static void slider_init(gpointer widget, const gchar *key, UIData *ui)
 
 WidgetData *slider_register(SkinData *skin, SliderData *slider, const gchar *key, const gchar *text_id)
 {
-        return skin_register_widget(skin, key, text_id, type_id, slider);
+	WidgetData *wd;
+
+	wd = skin_register_widget(skin, key, text_id, type_id, slider);
+	slider->wd = wd;
+	slider->skin = skin;
+
+	return wd;
+}
+
+WidgetData *slider_register_to_skin(const gchar *key, gchar **data, gint x, gint y,
+				    gint length, gint vertical, gint reversed,
+				    gint has_press, gint prelight,
+				    const gchar *item_key,
+				    SkinData *skin, const gchar *text_id)
+{
+	SliderData *slider;
+
+	slider = slider_new_from_data(data, x, y, length, vertical, reversed,
+				      has_press, prelight, item_key);
+	return slider_register(skin, slider, key, text_id);
 }
 
 RegisterData *slider_register_key(const gchar *key, UIData *ui,
@@ -657,6 +874,8 @@ RegisterData *slider_register_key(const gchar *key, UIData *ui,
 {
 	SliderCallbackData *cd;
 
+	slider_type_init();
+
 	cd = g_new0(SliderCallbackData, 1);
 
 	cd->status_get_func = status_get_func;
@@ -694,11 +913,7 @@ static void slider_value_set_cb(WidgetData *wd, gpointer data, GdkPixbuf *pb, UI
 
 gint slider_value_set(const gchar *key, UIData *ui, gfloat value)
 {
-	gint ret;
-
-	ret = skin_widget_for_each_key(ui, key, type_id, slider_value_set_cb, &value);
-
-	return ret;
+	return skin_widget_for_each_key(ui, key, type_id, slider_value_set_cb, &value);
 }
 
 gfloat slider_value_get(const gchar *key, UIData *ui)
@@ -713,6 +928,27 @@ gfloat slider_value_get(const gchar *key, UIData *ui)
 	return slider_value_public(slider);
 }
 
+static void slider_step_size_set_cb(WidgetData *wd, gpointer data, GdkPixbuf *pb, UIData *ui)
+{
+	SliderData *slider;
+	SliderSetData *sd = data;
+
+	slider = wd->widget;
+
+	slider->step_size = sd->step_size;
+	slider->page_size = sd->page_size;
+}
+
+gint slider_step_size_set(const gchar *key, UIData *ui, gfloat step_size, gint page_size)
+{
+	SliderSetData sd;
+
+	sd.step_size = step_size;
+	sd.page_size = page_size;
+
+	return skin_widget_for_each_key(ui, key, type_id, slider_step_size_set_cb, &sd);
+}
+
 /*
  *-----------------------------
  * init
@@ -741,6 +977,9 @@ void slider_type_init(void)
 	od->func_back = slider_back_set;
 	od->func_free = slider_free_cb;
 
+	od->func_focus_key_event = slider_key_event_cb;
+	od->func_focus_draw = slider_focus_draw_cb;
+
 	od->func_get_geometry = slider_get_geometry;
 	od->func_set_coord = slider_set_coord;
 	od->func_set_size = slider_set_size;
