@@ -42,20 +42,39 @@ struct _SliderData
 	gint border_1;	/* top or left border (dep. on vertical flag) */
 	gint border_2;	/* bottom or right border */
 	gint stretch;
+
+	gfloat step_size;	/* amount to increment one step */
+	gint page_size;		/* number of steps (above) in one page */
+
+	gint key_timer_id;
+
+	WidgetData *wd;
 };
 
 
-SliderData *slider_new(GdkPixbuf *pb, gint has_press, gint prelight, gint vertical, gint reversed,
-		       gint length, gint x, gint y, const gchar *item_key);
-SliderData *slider_new_from_data(gchar **data, gint has_press, gint prelight, gint vertical, gint reversed,
-				 gint length, gint x, gint y, const gchar *item_key);
-SliderData *slider_new_from_file(const gchar *file, gint has_press, gint prelight, gint vertical, gint reversed,
-			gint length, gint x, gint y, const gchar *item_key);
+SliderData *slider_new(GdkPixbuf *pb, gint x, gint y,
+		       gint length, gint vertical, gint reversed,
+		       gint has_press, gint prelight,
+		       const gchar *item_key);
+SliderData *slider_new_from_data(gchar **data, gint x, gint y,
+				 gint length, gint vertical, gint reversed,
+				 gint has_press, gint prelight,
+				 const gchar *item_key);
+SliderData *slider_new_from_file(const gchar *file, gint x, gint y,
+				 gint length, gint vertical, gint reversed,
+				 gint has_press, gint prelight,
+				 const gchar *item_key);
 void slider_set_attributes(SliderData *slider, gint sizeable,
 			   gint real_length, gint border_1, gint border_2, gint stretch);
 void slider_free(SliderData *slider);
 
 WidgetData *slider_register(SkinData *skin, SliderData *slider, const gchar *key, const gchar *text_id);
+WidgetData *slider_register_to_skin(const gchar *key, gchar **data, gint x, gint y,
+				    gint length, gint vertical, gint reversed,
+				    gint has_press, gint prelight,
+				    const gchar *item_key,
+				    SkinData *skin, const gchar *text_id);
+
 RegisterData *slider_register_key(const gchar *key, UIData *ui,
 				  gfloat (*status_get_func)(SliderData *slider, const gchar *key, gpointer data), gpointer status_get_data,
 				  void (*slider_press_func)(SliderData *slider, const gchar *key, gfloat value, gpointer data), gpointer slider_press_data,
@@ -64,6 +83,8 @@ RegisterData *slider_register_key(const gchar *key, UIData *ui,
 gint slider_value_set(const gchar *key, UIData *ui, gfloat value);
 gfloat slider_value_get(const gchar *key, UIData *ui);
 
+gint slider_step_size_set(const gchar *key, UIData *ui, gfloat step_size, gint page_size);
+
 
 WidgetType slider_type_id(void);
 void slider_type_init(void);
