@@ -318,7 +318,7 @@ static void slider_edit_props_reversed_cb(GtkWidget *button, gpointer data)
 static void slider_edit_props_item_key_cb(GtkWidget *entry, gpointer data)
 {
 	SliderDetail *sd = data;
-	gchar *buf;
+	const gchar *buf;
 
 	g_free(sd->slider->item_key);
 	buf = gtk_entry_get_text(GTK_ENTRY(entry));
@@ -363,27 +363,34 @@ static gpointer slider_edit_props(UIData *ui, WidgetData *wd, GtkWidget *vbox, g
 		hbox = ui_edit_frame_new(vbox2, FALSE, NULL);
 
 		sd->real_length_spin = ui_edit_spin_new(hbox, _("Length:"), 1, SKIN_SIZE_MAX, &adj);
-		gtk_signal_connect(adj, "value_changed", slider_edit_props_real_length_cb, sd);
+		gtk_signal_connect(adj, "value_changed",
+				   GTK_SIGNAL_FUNC(slider_edit_props_real_length_cb), sd);
 
 		sd->stretch_button = ui_edit_toggle_new(hbox, _("Stretch"));
-		gtk_signal_connect(GTK_OBJECT(sd->stretch_button), "clicked", slider_edit_props_stretch_cb, sd);
+		gtk_signal_connect(GTK_OBJECT(sd->stretch_button), "clicked",
+				   GTK_SIGNAL_FUNC(slider_edit_props_stretch_cb), sd);
 
 		hbox = ui_edit_frame_new(vbox2, FALSE, NULL);
 
 		sd->border_1_spin = ui_edit_spin_new(hbox, _("Border 1:"), 0, SKIN_SIZE_MAX, &adj);
-		gtk_signal_connect(adj, "value_changed", slider_edit_props_border_1_cb, sd);
+		gtk_signal_connect(adj, "value_changed",
+				   GTK_SIGNAL_FUNC(slider_edit_props_border_1_cb), sd);
 
 		sd->border_2_spin = ui_edit_spin_new(hbox, _("Border 2:"), 0, SKIN_SIZE_MAX, &adj);
-		gtk_signal_connect(adj, "value_changed", slider_edit_props_border_2_cb, sd);
+		gtk_signal_connect(adj, "value_changed",
+				   GTK_SIGNAL_FUNC(slider_edit_props_border_2_cb), sd);
 
 		sd->sizeable_button = ui_edit_toggle_new(vbox, _("Dynamic size"));
-		gtk_signal_connect(GTK_OBJECT(sd->sizeable_button), "clicked", slider_edit_props_sizeable_cb, sd);
+		gtk_signal_connect(GTK_OBJECT(sd->sizeable_button), "clicked",
+				   GTK_SIGNAL_FUNC(slider_edit_props_sizeable_cb), sd);
 
 		sd->reverse_button = ui_edit_toggle_new(vbox, _("Reverse"));
-		gtk_signal_connect(GTK_OBJECT(sd->reverse_button), "clicked", slider_edit_props_reversed_cb, sd);
+		gtk_signal_connect(GTK_OBJECT(sd->reverse_button), "clicked",
+				   GTK_SIGNAL_FUNC(slider_edit_props_reversed_cb), sd);
 
 		sd->item_key_entry = ui_edit_entry_new(vbox, _("Item link key:") );
-		gtk_signal_connect(GTK_OBJECT(sd->item_key_entry), "changed", slider_edit_props_item_key_cb, sd);
+		gtk_signal_connect(GTK_OBJECT(sd->item_key_entry), "changed",
+				   GTK_SIGNAL_FUNC(slider_edit_props_item_key_cb), sd);
 		}
 
 	sd->slider = slider;
@@ -428,12 +435,13 @@ static void slider_edit_page_add_cb(GtkWidget *widget, gpointer data)
 		return;
 		}
 
-	slider = slider_new_from_file(image, ui_edit_toggle_get(pd->has_press_button),
-				      ui_edit_toggle_get(pd->has_prelight_button),
+	slider = slider_new_from_file(image, 0, 0,
+				      ui_edit_spin_get(pd->length_spin),
 				      ui_edit_toggle_get(pd->vertical_button),
 				      ui_edit_toggle_get(pd->reverse_button),
-				      ui_edit_spin_get(pd->length_spin),
-				      0, 0, item_key);
+				      ui_edit_toggle_get(pd->has_press_button),
+				      ui_edit_toggle_get(pd->has_prelight_button),
+				      item_key);
 	slider_set_attributes(slider, ui_edit_toggle_get(pd->sizeable_button),
 			      ui_edit_spin_get(pd->real_length_spin),
 			      ui_edit_spin_get(pd->border_1_spin),
@@ -546,11 +554,13 @@ static GtkWidget *slider_edit_page_new(EditData *ed)
 
 	button = gtk_button_new_with_label(_("Add"));
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", slider_edit_page_add_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(slider_edit_page_add_cb), pd);
 	gtk_widget_show(button);
 
 	pd->clist = ui_edit_clist_new(hbox, titles, 5);
-	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row", slider_edit_page_clist_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row",
+			   GTK_SIGNAL_FUNC(slider_edit_page_clist_cb), pd);
 
 	gtk_widget_show(hbox);
 
