@@ -64,6 +64,8 @@ FontData *font_new(GdkPixbuf *pb, gint extended)
 	gint width;
 	gint height;
 
+	font_type_init();
+
 	if (!pb) return NULL;
 
 	font = g_new0(FontData, 1);
@@ -237,6 +239,16 @@ WidgetData *font_register(SkinData *skin, FontData *font, const gchar *key, cons
 	return skin_register_widget(skin, key, text_id, type_id_font, font);
 }
 
+WidgetData *font_register_to_skin(const gchar *key, gchar **data, 
+				  gint extended,
+				  SkinData *skin, const gchar *text_id)
+{
+	FontData *font;
+
+	font = font_new_from_data(data, extended);
+	return font_register(skin, font, key, text_id);
+}
+
 /*
  *-----------------------------
  * init
@@ -284,10 +296,14 @@ static void text_scroll_update_set(TextData *text, gint enable);
  *-----------------------------
  */
 
-TextData *text_new(FontData *font, gint length, gint sizeable, gint x, gint y)
+TextData *text_new(FontData *font, gint x, gint y,
+		   gint length, gint sizeable)
 {
 	TextData *text;
 
+	font_type_init();
+	text_type_init();
+
 	if (!font || length < 1) return NULL;
 
 	util_size(&x);
@@ -318,26 +334,28 @@ TextData *text_new(FontData *font, gint length, gint sizeable, gint x, gint y)
 	return text;
 }
 
-TextData *text_new_from_data(gchar **data, gint length, gint extended, gint sizeable, gint x, gint y)
+TextData *text_new_from_data(gchar **data, gint x, gint y,
+			     gint length, gint sizeable, gint extended)
 {
 	TextData *text;
 	FontData *font;
 
 	font = font_new_from_data(data, extended);
 
-	text = text_new(font, length, sizeable, x, y);
+	text = text_new(font, x, y, length, sizeable);
 	font_unref(font);
 	return text;
 }
 
-TextData *text_new_from_file(const gchar *file, gint length, gint extended, gint sizeable, gint x, gint y)
+TextData *text_new_from_file(const gchar *file, gint x, gint y,
+			    gint length, gint sizeable, gint extended)
 {
 	TextData *text;
 	FontData *font;
 
 	font = font_new_from_file(file, extended);
 
-	text = text_new(font, length, sizeable, x, y);
+	text = text_new(font, x, y, length, sizeable);
 	font_unref(font);
 	return text;
 }
@@ -404,7 +422,7 @@ static void text_draw_real(TextData *text, GdkPixbuf *pb, UIData *ui)
 		font_draw(text->font, c, text->x, text->y, i, pb);
 		}
 
-	ui_display_render_area(ui, text->x, text->y, text->width, text->height);
+	ui_display_render_area(ui, text->x, text->y, text->width, text->height, NULL);
 	text_scroll_update_set(text, TRUE);
 }
 
@@ -636,7 +654,7 @@ static WidgetData *text_parse(SkinData *skin, GList *list, const gchar *skin_dir
 
 	if (filename)
 		{
-		text = text_new_from_file(filename, length, extended, sizeable, x, y);
+		text = text_new_from_file(filename, x, y, length, sizeable, extended);
 		}
 	else
 		{
@@ -651,7 +669,7 @@ static WidgetData *text_parse(SkinData *skin, GList *list, const gchar *skin_dir
 			}
 
 		font = font_wd->widget;
-		text = text_new(font, length, sizeable, x, y);
+		text = text_new(font, x, y, length, sizeable);
 		}
 
 	if (text)
@@ -681,11 +699,24 @@ WidgetData *text_register(SkinData *skin, TextData *text, const gchar *key, cons
 	return skin_register_widget(skin, key, text_id, type_id, text);
 }
 
+WidgetData *text_register_to_skin(const gchar *key, gchar **data, gint x, gint y,
+				  gint length, gint sizeable, gint extended,
+				  SkinData *skin, const gchar *text_id)
+{
+	TextData *text;
+
+	text = text_new_from_data(data, x, y, length, sizeable, extended);
+	return text_register(skin, text, key, text_id);
+}
+
 RegisterData *text_register_key(const gchar *key, UIData *ui,
 				const gchar *(*status_get_func)(TextData *text, const gchar *key, gpointer data), gpointer status_get_data)
 {
 	TextCallbackData *cd;
 
+	font_type_init();
+	text_type_init();
+
 	cd = g_new0(TextCallbackData, 1);
 
 	cd->status_get_func = status_get_func;
@@ -765,6 +796,8 @@ void text_type_init(void)
 
 	if (type_id != -1) return;
 
+	font_type_init();
+
 	od = ui_widget_type_new("text");
 	type_id = od->type;
 
@@ -783,5 +816,3 @@ void text_type_init(void)
 	text_type_init_edit(od);
 }
 
-
-
