@@ -332,10 +332,11 @@ static gpointer text_edit_props(UIData *ui, WidgetData *wd, GtkWidget *vbox, gpo
 
 		td->sizeable_button = ui_edit_toggle_new(vbox, _("Dynamic size"));
 		gtk_signal_connect(GTK_OBJECT(td->sizeable_button), "clicked",
-				   text_edit_props_sizeable_cb, td);
+				   GTK_SIGNAL_FUNC(text_edit_props_sizeable_cb), td);
 
 		td->spin_length = ui_edit_spin_new(vbox, _("length:"), 1, 200, &adj);
-		gtk_signal_connect(adj, "value_changed", text_edit_props_length_cb, td);
+		gtk_signal_connect(adj, "value_changed",
+				   GTK_SIGNAL_FUNC(text_edit_props_length_cb), td);
 
 		hbox = ui_edit_frame_new(vbox, FALSE, NULL);
 
@@ -418,15 +419,17 @@ static void text_edit_page_add_cb(GtkWidget *widget, gpointer data)
 
 		wd = skin_widget_get_by_key(pd->ed->ui->skin, font_id, font_type_id());
 		font = wd->widget;
-		text = text_new(font, ui_edit_spin_get(pd->length_spin),
-				ui_edit_toggle_get(pd->sizeable_button), 0, 0);
+		text = text_new(font, 0, 0,
+				ui_edit_spin_get(pd->length_spin),
+				ui_edit_toggle_get(pd->sizeable_button));
 		image = NULL;
 		}
 	else
 		{
-		text = text_new_from_file(image, ui_edit_spin_get(pd->length_spin),
-					  ui_edit_toggle_get(pd->extended_button),
-					  ui_edit_toggle_get(pd->sizeable_button), 0, 0);
+		text = text_new_from_file(image, 0, 0,
+					  ui_edit_spin_get(pd->length_spin),
+					  ui_edit_toggle_get(pd->sizeable_button),
+					  ui_edit_toggle_get(pd->extended_button));
 		}
 	if (text)
 		{
@@ -505,14 +508,16 @@ static GtkWidget *text_edit_page_new(EditData *ed)
 	pd->length_spin = ui_edit_spin_new(vbox, _("Length:"), 1, 200, NULL);
 
 	pd->font_id_entry = ui_edit_entry_new(vbox, _("Font key:"));
-	gtk_signal_connect(GTK_OBJECT(pd->font_id_entry), "changed", text_edit_page_font_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(pd->font_id_entry), "changed",
+			   GTK_SIGNAL_FUNC(text_edit_page_font_cb), pd);
 	pd->image_entry = ui_edit_path_entry_new(vbox, _("Image:"), "SLIK_text_image");
 	pd->pixmap = ui_edit_pixmap_new(vbox);
 	ui_edit_path_entry_connect_pixmap(pd->image_entry, pd->pixmap);
 
 	button = gtk_button_new_with_label(_("Add"));
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", text_edit_page_add_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(text_edit_page_add_cb), pd);
 	gtk_widget_show(button);
 
 	sep = gtk_hseparator_new();
@@ -524,14 +529,19 @@ static GtkWidget *text_edit_page_new(EditData *ed)
 
 	button = gtk_button_new_with_label(_("Add"));
 	gtk_box_pack_end(GTK_BOX(frame), button, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", text_edit_page_font_add_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(text_edit_page_font_add_cb), pd);
 	gtk_widget_show(button);
 
 	pd->clist = ui_edit_clist_new(hbox, titles, 3);
-	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row", text_edit_page_clist_cb, pd);
+	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row",
+			   GTK_SIGNAL_FUNC(text_edit_page_clist_cb), pd);
 
 	gtk_widget_show(hbox);
 
+	/* hacky, but it works */
+	ui_edit_widget_id_list_set_page(ed->widget_id_list, font_type_id(), hbox);
+
 	return hbox;
 }
 
