@@ -30,16 +30,45 @@
 
 void util_size(gint *n)
 {
-	if (slik_double_size) *n *= DOUBLE_MULTIPLIER;
+	if (slik_scale && slik_scale_value > 0.1)
+		{
+		if (*n > 0)
+			{
+			*n = (gint)((gfloat)*n * slik_scale_value);
+			if (*n < 1) *n = 1;
+			}
+		}
+	else if (slik_double_size)
+		{
+		*n *= DOUBLE_MULTIPLIER;
+		}
 }
 
 GdkPixbuf *util_size_pixbuf(GdkPixbuf *pb, gint apply_adjustments)
 {
-	if (pb && slik_double_size)
+	if (slik_scale && slik_scale_value > 0.1)
+		{
+		gint w, h;
+		GdkPixbuf *tmp;
+
+		tmp = pb;
+		w = gdk_pixbuf_get_width(tmp);
+		h = gdk_pixbuf_get_height(tmp);
+		w = (gint)((gfloat)w * slik_scale_value);
+		if (w < 1) w = 1;
+		h = (gint)((gfloat)h * slik_scale_value);
+		if (h < 1) h = 1;
+
+		pb = gdk_pixbuf_scale_simple(tmp, w, h, GDK_INTERP_NEAREST);
+
+		gdk_pixbuf_unref(tmp);
+		}
+	else if (pb && slik_double_size)
 		{
 		GdkPixbuf *tmp = pb;
 		pb = gdk_pixbuf_scale_simple(tmp, gdk_pixbuf_get_width(tmp) * DOUBLE_MULTIPLIER,
-					     gdk_pixbuf_get_height(tmp) * DOUBLE_MULTIPLIER, GDK_INTERP_NEAREST);
+					     gdk_pixbuf_get_height(tmp) * DOUBLE_MULTIPLIER,
+					     GDK_INTERP_NEAREST);
 		gdk_pixbuf_unref(tmp);
 		}
 
@@ -69,6 +98,133 @@ GdkPixbuf *util_size_pixbuf(GdkPixbuf *pb, gint apply_adjustments)
 	return pb;
 }
 
+gint util_clip_region(gint x, gint y, gint w, gint h,
+		      gint clip_x, gint clip_y, gint clip_w, gint clip_h,
+		      gint *rx, gint *ry, gint *rw, gint *rh)
+{
+	if (clip_x + clip_w < x ||
+	    clip_x > x + w ||
+	    clip_y + clip_h < y ||
+	    clip_y > y + h)
+		{
+		return FALSE;
+		}
+
+	*rx = MAX(x, clip_x);
+	*rw = MIN((x + w), (clip_x + clip_w)) - *rx;
+
+	*ry = MAX(y, clip_y);
+	*rh = MIN((y + h), (clip_y + clip_h)) - *ry;
+
+	return TRUE;
+}
+
+void util_pixbuf_copy_border_clipped(GdkPixbuf *src,
+				     gint border_left, gint border_right,
+				     gint border_top, gint border_bottom,
+				     GdkPixbuf *dest, gint x, gint y, gint w, gint h,
+				     gint clip_x, gint clip_y, gint clip_w, gint clip_h,
+				     gint alpha)
+{
+	gint bx, by, bw, bh;
+	gint sw, sh;
+	gint rx, ry;
+
+	sw = gdk_pixbuf_get_width(src);
+	sh = gdk_pixbuf_get_height(src);
+
+	border_left = MIN(border_left, w);
+	border_right = MIN(border_right, w);
+	border_top = MIN(border_top, h);
+	border_bottom = MIN(border_bottom, h);
+
+	/* top left */
+	rx = x;
+	ry = y;
+	if (util_clip_region(rx, ry, border_left, border_top,
+			     clip_x, clip_y, clip_w, clip_h,
+			     &bx, &by, &bw, &bh))
+		{
+		pixbuf_copy_area_alpha(src, bx - rx, by - ry,
+				       dest, bx, by, bw, bh, alpha);
+		}
+	/* top */
+	rx = x + border_left;
+	ry = y;
+	if (util_clip_region(rx, ry, w - border_left - border_right, border_top,
+			     clip_x, clip_y, clip_w, clip_h,
+			     &bx, &by, &bw, &bh))
+		{
+		pixbuf_copy_fill_alpha_offset(src, border_left, 0, sw - border_left - border_right, border_top,
+					      dest, bx, by, bw, bh, FALSE, alpha,
+					      bx - rx, by - ry);
+		}
+	/* top right */
+	rx = x + w - border_right;
+	ry = y;
+	if (util_clip_region(rx, ry, border_right, border_top,
+			     clip_x, clip_y, clip_w, clip_h,
+			     &bx, &by, &bw, &bh))
+		{
+		pixbuf_copy_area_alpha(src, sw - border_right + (bx - rx), by - ry,
+				       dest, bx, by, bw, bh, alpha);
+		}
+	/* left */
+	rx = x;
+	ry = y + border_top;
+	if (util_clip_region(rx, ry, border_left, h - border_top - border_bottom,
+			     clip_x, clip_y, clip_w, clip_h,
+			     &bx, &by, &bw, &bh))
+		{
+		pixbuf_copy_fill_alpha_offset(src, 0, border_top, border_left, sh - border_top - border_bottom,
+					      dest, bx, by, bw, bh, FALSE, alpha,
+					      bx - rx, by - ry);
+		}
+	/* right */
+	rx = x + w - border_right;
+	ry = y + border_top;
+	if (util_clip_region(rx, ry, border_right, h - border_top - border_bottom,
+			     clip_x, clip_y, clip_w, clip_h,
+			     &bx, &by, &bw, &bh))
+		{
+		pixbuf_copy_fill_alpha_offset(src, sw - border_right, border_top, border_right, sh - border_top - border_bottom,
+					      dest, bx, by, bw, bh, FALSE, alpha,
+					      bx - rx, by - ry);
+		}
+	/* bottom left */
+	rx = x;
+	ry = y + h - border_bottom;
+	if (util_clip_region(rx, ry, border_left, border_bottom,
+			     clip_x, clip_y, clip_w, clip_h,
+			     &bx, &by, &bw, &bh))
+		{
+		pixbuf_copy_area_alpha(src, bx - rx, sh - border_bottom + (by - ry),
+				       dest, bx, by, bw, bh, alpha);
+		}
+	/* bottom */
+	rx = x + border_left;
+	ry = y + h - border_bottom;
+	if (util_clip_region(rx, ry, w - border_left - border_right, border_bottom,
+			     clip_x, clip_y, clip_w, clip_h,
+			     &bx, &by, &bw, &bh))
+		{
+		pixbuf_copy_fill_alpha_offset(src, border_left, sh - border_bottom, sw - border_left - border_right, border_bottom,
+					      dest, bx, by, bw, bh, FALSE, alpha,
+					      bx - rx, by - ry);
+		}
+	/* bottom right */
+	rx = x + w - border_right;
+	ry = y + h - border_bottom;
+	if (util_clip_region(rx, ry, border_right, border_bottom,
+			     clip_x, clip_y, clip_w, clip_h,
+			     &bx, &by, &bw, &bh))
+		{
+		pixbuf_copy_area_alpha(src, sw - border_right + (bx - rx), sh - border_bottom + (by - ry),
+				       dest, bx, by, bw, bh, alpha);
+		}
+}
+
+
 /* 
  * returns a portion of the root window in a gdk_pixbuf
  * any part that is off the screen is set to black.
@@ -130,7 +286,7 @@ gint util_pixbuf_fill_from_root_window(GdkPixbuf *pb, gint x, gint y, gint get_a
 
 	if (w < 1 || h < 1) return TRUE;
 
-	rootwindow = (GdkWindow *) &gdk_root_parent;;
+	rootwindow = GDK_ROOT_PARENT();
 	if (!rootwindow) return TRUE;
 
 	gdkvisual = gdk_window_get_visual(rootwindow);
@@ -170,13 +326,13 @@ gint util_pixbuf_fill_from_root_window(GdkPixbuf *pb, gint x, gint y, gint get_a
 
 				if (p != None)
 					{
-					GdkPixmap *pp;
+					static GdkPixmap *pp = NULL;
 					GdkColormap *cmap;
 					gint p_w, p_h;
 
 					clear_window = FALSE;
 
-					pp = gdk_pixmap_foreign_new(p);
+					if (!pp) pp = gdk_pixmap_foreign_new(p);
 					cmap = gdk_window_get_colormap(rootwindow);
 
 					gdk_window_get_size(pp, &p_w, &p_h);
@@ -227,7 +383,6 @@ gint util_pixbuf_fill_from_root_window(GdkPixbuf *pb, gint x, gint y, gint get_a
 									     screen_x, screen_y, 
 								     	     screen_x - x, screen_y - y, w, h);
 						}
-					g_free(pp);
 					}
 				}
 			if (data) XFree(data);
@@ -519,3 +674,5 @@ gint ui_state_get_active(const gchar *skin_name, const gchar *window_id)
 	return ws->active;
 }
 
+
+
