@@ -17,6 +17,17 @@ void util_size(gint *n);
 
 GdkPixbuf *util_size_pixbuf(GdkPixbuf *pb, gint apply_adjustments);
 
+gint util_clip_region(gint x, gint y, gint w, gint h,
+		      gint clip_x, gint clip_y, gint clip_w, gint clip_h,
+		      gint *rx, gint *ry, gint *rw, gint *rh);
+
+void util_pixbuf_copy_border_clipped(GdkPixbuf *src,
+				     gint border_left, gint border_right,
+				     gint border_top, gint border_bottom,
+				     GdkPixbuf *dest, gint x, gint y, gint w, gint h,
+				     gint clip_x, gint clip_y, gint clip_w, gint clip_h,
+				     gint alpha);
+
 gint util_pixbuf_fill_from_root_window(GdkPixbuf *pb, gint x, gint y, gint get_all);
 GdkPixbuf *util_pixbuf_from_root_window(gint x, gint y, gint w, gint h, gint get_all);
 
