@@ -66,7 +66,7 @@ static void clist_edit_make_row_visible(ClistEditData *ced)
 		fcol = 1.0;
 		}
 
-	if (ncol > 0 || fcol > 0) gtk_clist_moveto(ced->clist, nrow, ncol, frow, fcol);
+	if (ncol >= 0 || nrow >= 0) gtk_clist_moveto(ced->clist, nrow, ncol, frow, fcol);
 }
 
 static void clist_edit_close(ClistEditData *ced)
