@@ -731,6 +731,7 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 	GtkWidget *scrolled;
 	GtkWidget *button;
 	GtkWidget *table;
+	GtkWidget *paned;
 #ifdef PATH_SEL_USE_HEADINGS
 	gchar *d_title[] = { "Directories", NULL };
 	gchar *f_title[] = { "Files", NULL };
@@ -775,8 +776,20 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 	scrolled = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
 				GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
-	gtk_table_attach(GTK_TABLE(table), scrolled, 0, 1, 1, 2,
-			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	if (filter)
+		{
+		paned = gtk_hpaned_new();
+		gtk_table_attach(GTK_TABLE(table), paned, 0, 3, 1, 2,
+				 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+		gtk_widget_show(paned);
+		gtk_paned_add1(GTK_PANED(paned), scrolled);
+		}
+	else
+		{
+		paned = NULL;
+		gtk_table_attach(GTK_TABLE(table), scrolled, 0, 1, 1, 2,
+				 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+		}
 	gtk_widget_show(scrolled);
 
 #ifdef PATH_SEL_USE_HEADINGS
@@ -816,8 +829,15 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 		scrolled = gtk_scrolled_window_new(NULL, NULL);
 		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
 					GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
-		gtk_table_attach(GTK_TABLE(table), scrolled, 2, 3, 1, 2,
+		if (paned)
+			{
+			gtk_paned_add2(GTK_PANED(paned), scrolled);
+			}
+		else
+			{
+			gtk_table_attach(GTK_TABLE(table), scrolled, 2, 3, 1, 2,
 				 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+			}
 		gtk_widget_show(scrolled);
 
 #ifdef PATH_SEL_USE_HEADINGS
@@ -931,6 +951,3 @@ void path_selection_clear_filter(GtkWidget *entry)
 	dest_filter_clear(dd);
 }
 
-
-
-
