@@ -143,11 +143,12 @@ void pixbuf_copy_area_alpha(GdkPixbuf *src, gint sx, gint sy,
  */
 static void _pixbuf_copy_fill_alpha(GdkPixbuf *src, gint sx, gint sy, gint sw, gint sh,
 				    GdkPixbuf *dest, gint dx, gint dy, gint dw, gint dh,
-				    gint stretch, gint apply_alpha, gint alpha_modifier, gint alpha_too)
+				    gint stretch, gint apply_alpha, gint alpha_modifier, gint alpha_too,
+				    gint offset_x, gint offset_y)
 {
-	if (sh < 1 || sw < 1 || dw < 1 || dw < 1) return;
+	if (sw < 1 || sh < 1 || dw < 1 || dh < 1) return;
 
-	if (sw == dw && sh == dh)
+	if (sw == dw && sh == dh && offset_x == 0 && offset_y == 0)
 		{
 		_pixbuf_copy_area_alpha(src, sx, sy, dest, dx, dy, sw, sh,
 					apply_alpha, alpha_modifier, alpha_too);
@@ -160,6 +161,7 @@ static void _pixbuf_copy_fill_alpha(GdkPixbuf *src, gint sx, gint sy, gint sw, g
 
 		/* uh, is it this hard to figure out the offset? or is pixbuf_scale just plain broken
 		 * when offets are not 0 ?
+		 * (also note stretch does not support offsets)
 		 */
 
 		if (apply_alpha)
@@ -183,21 +185,34 @@ static void _pixbuf_copy_fill_alpha(GdkPixbuf *src, gint sx, gint sy, gint sw, g
 
 		/* tile */
 
-		if (sw > dw) sw = dw;
-		if (sh > dh) sh = dh;
+		if (offset_x != 0) offset_x = offset_x % sw;
+		if (offset_y != 0) offset_y = offset_y % sh;
 
 		for (x = dx; x < dx + dw; x += sw)
 		    for (y = dy; y < dy + dh; y += sh)
 			{
 			gint w, h;
+			gint ox, oy;
 
 			w = sw;
 			h = sh;
+			ox = sx;
+			oy = sy;
+			if (x == dx && offset_x > 0)
+				{
+				ox += offset_x;
+				w -= offset_x;
+				}
+			if (y == dy && offset_y > 0)
+				{
+				oy += offset_y;
+				h -= offset_y;
+				}
 			if (x + w > dx + dw) w = dx + dw - x;
 			if (y + h > dy + dh) h = dy + dh - y;
 			if (w > 0 && h > 0)
 				{
-				_pixbuf_copy_area_alpha(src, sx, sy, dest, x, y, w, h,
+				_pixbuf_copy_area_alpha(src, ox, oy, dest, x, y, w, h,
 							apply_alpha, alpha_modifier, alpha_too);
 				}
 			}
@@ -210,7 +225,7 @@ void pixbuf_copy_fill(GdkPixbuf *src, gint sx, gint sy, gint sw, gint sh,
 {
 	_pixbuf_copy_fill_alpha(src, sx, sy, sw, sh,
 				dest, dx, dy, dw, dh,
-				stretch, FALSE, 0, alpha_too);
+				stretch, FALSE, 0, alpha_too, 0, 0);
 }
 
 void pixbuf_copy_fill_alpha(GdkPixbuf *src, gint sx, gint sy, gint sw, gint sh,
@@ -219,7 +234,18 @@ void pixbuf_copy_fill_alpha(GdkPixbuf *src, gint sx, gint sy, gint sw, gint sh,
 {
 	_pixbuf_copy_fill_alpha(src, sx, sy, sw, sh,
 				dest, dx, dy, dw, dh,
-				stretch, TRUE, alpha_modifier, FALSE);
+				stretch, TRUE, alpha_modifier, FALSE, 0, 0);
+}
+
+void pixbuf_copy_fill_alpha_offset(GdkPixbuf *src, gint sx, gint sy, gint sw, gint sh,
+				   GdkPixbuf *dest, gint dx, gint dy, gint dw, gint dh,
+				   gint stretch, gint alpha_modifier,
+				   gint offset_x, gint offset_y)
+{
+	_pixbuf_copy_fill_alpha(src, sx, sy, sw, sh,
+				dest, dx, dy, dw, dh,
+				stretch, TRUE, alpha_modifier, FALSE,
+				offset_x, offset_y);
 }
 
 static void _pixbuf_copy_fill_border(GdkPixbuf *src, GdkPixbuf *dest,
@@ -267,25 +293,25 @@ static void _pixbuf_copy_fill_border(GdkPixbuf *src, GdkPixbuf *dest,
 	_pixbuf_copy_fill_alpha(src, 0, border_top, border_left, sh - n,
 				dest, 0, border_top, border_left, dh - n,
 				left_stretch,
-				apply_alpha, alpha_modifier, alpha_too);
+				apply_alpha, alpha_modifier, alpha_too, 0, 0);
 	_pixbuf_copy_fill_alpha(src, sw - border_right, border_top, border_right, sh - n,
 				dest, dw - border_right, border_top, border_right, dh - n,
 				right_stretch,
-				apply_alpha, alpha_modifier, alpha_too);
+				apply_alpha, alpha_modifier, alpha_too, 0, 0);
 	_pixbuf_copy_fill_alpha(src, border_left, 0, sw - m, border_top,
 				dest, border_left, 0, dw - m, border_top,
 				top_stretch,
-				apply_alpha, alpha_modifier, alpha_too);
+				apply_alpha, alpha_modifier, alpha_too, 0, 0);
 	_pixbuf_copy_fill_alpha(src, border_left, sh - border_bottom, sw - m, border_bottom,
 				dest, border_left, dh - border_bottom, dw - m, border_bottom,
 				bottom_stretch,
-				apply_alpha, alpha_modifier, alpha_too);
+				apply_alpha, alpha_modifier, alpha_too, 0, 0);
 
 	/* center field */
 	_pixbuf_copy_fill_alpha(src, border_left, border_top, sw - m, sh - n,
 				dest, border_left, border_top, dw - m, dh - n,
 				stretch_center,
-				apply_alpha, alpha_modifier, alpha_too);
+				apply_alpha, alpha_modifier, alpha_too, 0, 0);
 }
 
 void pixbuf_copy_fill_border(GdkPixbuf *src, GdkPixbuf *dest,
@@ -365,7 +391,7 @@ void pixbuf_copy_channel(GdkPixbuf *src, gint sx, gint sy,
 	d_pix = gdk_pixbuf_get_pixels(dest);
 
 	if (channel < 0 ||
-	    (channel > 2 && !s_alpha && !d_alpha) ) return;
+	    (channel > 2 && (!s_alpha || !d_alpha)) ) return;
 	if (channel > 2) channel = 3;
 
 	s_step = s_alpha ? 4 : 3;
@@ -377,7 +403,9 @@ void pixbuf_copy_channel(GdkPixbuf *src, gint sx, gint sy,
 		dp = d_pix + (dy + i) * drs + (dx * d_step) + channel;
 		for (j = 0; j < w; j++)
 			{
-			*(dp += d_step) = *(sp += s_step);
+			*dp = *sp;
+			dp += d_step;
+			sp += s_step;
 			}
 		}
 	
@@ -932,8 +960,97 @@ gint pixbuf_pixel_is_visible(GdkPixbuf *pb, gint x, gint y)
 	return ( (na > 0) );
 }
 
+static void pixbuf_copy_point_real(GdkPixbuf *src, gint sx, gint sy,
+				   GdkPixbuf *dest, gint dx, gint dy,
+				   gint use_alpha, gint at_alpha, gint alpha)
+{
+	guint8 r, g, b, a;
 
+	if (sx < 0 || sx >= gdk_pixbuf_get_width(src)) return;
+	if (sy < 0 || sy >= gdk_pixbuf_get_height(src)) return;
 
+	if (dx < 0 || dx >= gdk_pixbuf_get_width(dest)) return;
+	if (dy < 0 || dy >= gdk_pixbuf_get_height(dest)) return;
+
+	buf_pixel_get(gdk_pixbuf_get_pixels(src), sx, sy, &r, &g, &b, &a, 
+		      gdk_pixbuf_get_has_alpha(src), gdk_pixbuf_get_rowstride(src));
+
+	if (at_alpha)
+		{
+		buf_pixel_set(gdk_pixbuf_get_pixels(dest), dx, dy, r, g, b, alpha * a >> 8,
+			      gdk_pixbuf_get_has_alpha(dest), gdk_pixbuf_get_rowstride(dest));
+		}
+	else
+		{
+		buf_pixel_set(gdk_pixbuf_get_pixels(dest), dx, dy, r, g, b, use_alpha ? a : 255,
+			      gdk_pixbuf_get_has_alpha(dest), gdk_pixbuf_get_rowstride(dest));
+		}
+}
+
+void pixbuf_copy_point(GdkPixbuf *src, gint sx, gint sy,
+		       GdkPixbuf *dest, gint dx, gint dy, gint use_alpha)
+{
+	pixbuf_copy_point_real(src, sx, sy, dest, dx, dy, use_alpha, FALSE, 0);
+}
+
+void pixbuf_copy_point_at_alpha(GdkPixbuf *src, gint sx, gint sy,
+				GdkPixbuf *dest, gint dx, gint dy, gint alpha)
+{
+	pixbuf_copy_point_real(src, sx, sy, dest, dx, dy, FALSE, TRUE, alpha);
+}
+
+void pixbuf_copy_line(GdkPixbuf *src, gint sx1, gint sy1, gint sx2, gint sy2,
+		      GdkPixbuf *dest, gint dx, gint dy, gint use_alpha)
+{
+	guint8 r, g, b, a;
+	guchar *s_pix, *d_pix;
+	gint s_alpha, d_alpha;
+	gint srs, drs;
+
+	gint xd, yd;
+
+	gfloat xstep, ystep;
+
+	gfloat i, j;
+	gint n, nt;
+
+	if (sx1 == sx2 && sy1 == sy2);
+
+	if (sx1 < 0 || sx1 >= gdk_pixbuf_get_width(src)) return;
+	if (sy1 < 0 || sy1 >= gdk_pixbuf_get_height(src)) return;
+	if (sx2 < 0 || sx2 >= gdk_pixbuf_get_width(src)) return;
+	if (sy2 < 0 || sy2 >= gdk_pixbuf_get_height(src)) return;
+
+	if (dx < 0 || dx >= gdk_pixbuf_get_width(dest)) return;
+	if (dy < 0 || dy >= gdk_pixbuf_get_height(dest)) return;
+
+	xd = sx2 - sx1;
+	yd = sy2 - sy1;
+
+	if (dx + xd < 0 || dx + xd >= gdk_pixbuf_get_width(dest)) return;
+	if (dy + yd < 0 || dy + yd >= gdk_pixbuf_get_height(dest)) return;
+
+	s_pix = gdk_pixbuf_get_pixels(src);
+	srs = gdk_pixbuf_get_rowstride(src);
+	s_alpha = gdk_pixbuf_get_has_alpha(src);
+
+	d_pix = gdk_pixbuf_get_pixels(dest);
+	drs = gdk_pixbuf_get_rowstride(dest);
+	d_alpha = gdk_pixbuf_get_has_alpha(dest);
+
+	nt = sqrt(xd * xd + yd * yd);
+	xstep = (float)xd / nt;
+	ystep = (float)yd / nt;
+
+	i = j = 0.0;
+	for (n = 0; n < nt; n++)
+		{
+		buf_pixel_get(s_pix, sx1 + (gint)j, sy1 + (gint)i, &r, &g, &b, &a, s_alpha, srs);
+		buf_pixel_set(d_pix, dx + (gint)j, dy + (gint)i, r, g, b, use_alpha ? a : 255, d_alpha, drs);
+		i += ystep;
+		j += xstep;
+		}
+}
 
 
 
