@@ -503,7 +503,7 @@ static void tab_completion_popup_list(TabCompData *td, GList *list)
 		gchar *name = work->data;
 		GtkWidget *item;
 
-		item = menu_item_add(menu, name, tab_completion_popup_cb, name);
+		item = menu_item_add(menu, name, GTK_SIGNAL_FUNC(tab_completion_popup_cb), name);
 		gtk_object_set_data(GTK_OBJECT(item), "tab_completion_data", td);
 
 		work = work->next;
